@@ -55,8 +55,6 @@
 #include "nsStubImageDecoderObserver.h"
 #include "nsIPresShell.h"
 #include "nsPresContext.h"
-#include "nsIScrollableView.h"
-#include "nsIViewManager.h"
 #include "nsStyleContext.h"
 #include "nsAutoPtr.h"
 #include "nsMediaDocument.h"
@@ -73,6 +71,11 @@
 #include "nsIContentViewer.h"
 #include "nsIMarkupDocumentViewer.h"
 #include "nsIDocShellTreeItem.h"
+#include "nsThreadUtils.h"
+#include "nsIScrollableFrame.h"
+#include "mozilla/dom/Element.h"
+
+using namespace mozilla::dom;
 
 #define AUTOMATIC_IMAGE_RESIZING_PREF "browser.enable_automatic_image_resizing"
 #define CLICK_IMAGE_RESIZING_PREF "browser.enable_click_image_resizing"
@@ -87,7 +90,8 @@ public:
   ImageListener(nsImageDocument* aDocument);
   virtual ~ImageListener();
 
-  NS_DECL_NSIREQUESTOBSERVER
+  /* nsIRequestObserver */
+  NS_IMETHOD OnStartRequest(nsIRequest* request, nsISupports *ctxt);
 };
 
 class nsImageDocument : public nsMediaDocument,
@@ -120,6 +124,7 @@ public:
 
   // imgIDecoderObserver (override nsStubImageDecoderObserver)
   NS_IMETHOD OnStartContainer(imgIRequest* aRequest, imgIContainer* aImage);
+  NS_IMETHOD OnStopDecode(imgIRequest *aRequest, nsresult aStatus, const PRUnichar *aStatusArg);
 
   // nsIDOMEventListener
   NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent);
@@ -127,6 +132,10 @@ public:
   NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(nsImageDocument, nsMediaDocument)
 
   friend class ImageListener;
+
+  void DefaultCheckOverflowing() { CheckOverflowing(mResizeImageByDefault); }
+
+  virtual nsXPCClassInfo* GetClassInfo();
 protected:
   virtual nsresult CreateSyntheticDocument();
 
@@ -137,7 +146,7 @@ protected:
   nsresult ScrollImageTo(PRInt32 aX, PRInt32 aY, PRBool restoreImage);
 
   float GetRatio() {
-    return PR_MIN((float)mVisibleWidth / mImageWidth,
+    return NS_MIN((float)mVisibleWidth / mImageWidth,
                   (float)mVisibleHeight / mImageHeight);
   }
 
@@ -230,43 +239,6 @@ ImageListener::OnStartRequest(nsIRequest* request, nsISupports *ctxt)
   return nsMediaDocumentStreamListener::OnStartRequest(request, ctxt);
 }
 
-NS_IMETHODIMP
-ImageListener::OnStopRequest(nsIRequest* request, nsISupports *ctxt,
-                             nsresult status)
-{
-  NS_ENSURE_TRUE(mDocument, NS_ERROR_FAILURE);
-  nsImageDocument *imgDoc = (nsImageDocument*)mDocument.get();
-  imgDoc->UpdateTitleAndCharset();
-  
-  nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(imgDoc->mImageContent);
-  if (imageLoader) {
-    imgDoc->mObservingImageLoader = PR_FALSE;
-    imageLoader->RemoveObserver(imgDoc);
-  }
-
-  // |status| is NS_ERROR_PARSED_DATA_CACHED if the image was found in
-  // the cache (bug 177747 comment 51, bug 475344).
-  if (status == NS_ERROR_PARSED_DATA_CACHED) {
-    status = NS_OK;
-  }
-
-  // mImageContent can be null if the document is already destroyed
-  if (NS_FAILED(status) && imgDoc->mStringBundle && imgDoc->mImageContent) {
-    nsCAutoString src;
-    imgDoc->mDocumentURI->GetSpec(src);
-    NS_ConvertUTF8toUTF16 srcString(src);
-    const PRUnichar* formatString[] = { srcString.get() };
-    nsXPIDLString errorMsg;
-    NS_NAMED_LITERAL_STRING(str, "InvalidImage");
-    imgDoc->mStringBundle->FormatStringFromName(str.get(), formatString, 1,
-                                                getter_Copies(errorMsg));
-    
-    imgDoc->mImageContent->SetAttr(kNameSpaceID_None, nsGkAtoms::alt, errorMsg, PR_FALSE);
-  }
-
-  return nsMediaDocumentStreamListener::OnStopRequest(request, ctxt, status);
-}
-
 
   // NOTE! nsDocument::operator new() zeroes out all members, so don't
   // bother initializing members to 0.
@@ -296,7 +268,9 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 NS_IMPL_ADDREF_INHERITED(nsImageDocument, nsMediaDocument)
 NS_IMPL_RELEASE_INHERITED(nsImageDocument, nsMediaDocument)
 
-NS_INTERFACE_TABLE_HEAD(nsImageDocument)
+DOMCI_NODE_DATA(ImageDocument, nsImageDocument)
+
+NS_INTERFACE_TABLE_HEAD_CYCLE_COLLECTION_INHERITED(nsImageDocument)
   NS_HTML_DOCUMENT_INTERFACE_TABLE_BEGIN(nsImageDocument)
     NS_INTERFACE_TABLE_ENTRY(nsImageDocument, nsIImageDocument)
     NS_INTERFACE_TABLE_ENTRY(nsImageDocument, imgIDecoderObserver)
@@ -304,7 +278,7 @@ NS_INTERFACE_TABLE_HEAD(nsImageDocument)
     NS_INTERFACE_TABLE_ENTRY(nsImageDocument, nsIDOMEventListener)
   NS_OFFSET_AND_INTERFACE_TABLE_END
   NS_OFFSET_AND_INTERFACE_TABLE_TO_MAP_SEGUE
-  NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(ImageDocument)
+  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(ImageDocument)
 NS_INTERFACE_MAP_END_INHERITING(nsMediaDocument)
 
 
@@ -366,6 +340,11 @@ nsImageDocument::Destroy()
     if (mObservingImageLoader) {
       nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(mImageContent);
       if (imageLoader) {
+        // Push a null JSContext on the stack so that code that
+        // nsImageLoadingContent doesn't think it's being called by JS.  See
+        // Bug 631241
+        nsCxPusher pusher;
+        pusher.PushNull();
         imageLoader->RemoveObserver(this);
       }
     }
@@ -395,7 +374,7 @@ nsImageDocument::SetScriptGlobalObject(nsIScriptGlobalObject* aScriptGlobalObjec
   nsHTMLDocument::SetScriptGlobalObject(aScriptGlobalObject);
 
   if (aScriptGlobalObject) {
-    if (!GetRootContent()) {
+    if (!GetRootElement()) {
       // Create synthetic document
 #ifdef DEBUG
       nsresult rv =
@@ -462,6 +441,9 @@ nsImageDocument::GetImageRequest(imgIRequest** aImageRequest)
 NS_IMETHODIMP
 nsImageDocument::ShrinkToFit()
 {
+  if (!mImageContent) {
+    return NS_OK;
+  }
   if (GetZoomLevel() != mOriginalZoomLevel && mImageIsResized &&
       !nsContentUtils::IsChildOfSameType(this)) {
     return NS_OK;
@@ -470,8 +452,8 @@ nsImageDocument::ShrinkToFit()
   // Keep image content alive while changing the attributes.
   nsCOMPtr<nsIContent> imageContent = mImageContent;
   nsCOMPtr<nsIDOMHTMLImageElement> image = do_QueryInterface(mImageContent);
-  image->SetWidth(PR_MAX(1, NSToCoordFloor(GetRatio() * mImageWidth)));
-  image->SetHeight(PR_MAX(1, NSToCoordFloor(GetRatio() * mImageHeight)));
+  image->SetWidth(NS_MAX(1, NSToCoordFloor(GetRatio() * mImageWidth)));
+  image->SetHeight(NS_MAX(1, NSToCoordFloor(GetRatio() * mImageHeight)));
   
   // The view might have been scrolled when zooming in, scroll back to the
   // origin now that we're showing a shrunk-to-window version.
@@ -503,33 +485,27 @@ nsImageDocument::ScrollImageTo(PRInt32 aX, PRInt32 aY, PRBool restoreImage)
     FlushPendingNotifications(Flush_Layout);
   }
 
-  nsIPresShell *shell = GetPrimaryShell();
+  nsIPresShell *shell = GetShell();
   if (!shell)
     return NS_OK;
-  
-  nsIViewManager* vm = shell->GetViewManager();
-  if (!vm)
+
+  nsIScrollableFrame* sf = shell->GetRootScrollFrameAsScrollable();
+  if (!sf)
     return NS_OK;
 
-  nsIScrollableView* view;
-  vm->GetRootScrollableView(&view);
-  if (!view)
-    return NS_OK;
-
-  nsSize scrolledSize;
-  if (NS_FAILED(view->GetContainerSize(&scrolledSize.width, &scrolledSize.height)))
-    return NS_OK;
-
-  nsRect portRect = view->View()->GetBounds();
-  view->ScrollTo(nsPresContext::CSSPixelsToAppUnits(aX/ratio) - portRect.width/2,
-                 nsPresContext::CSSPixelsToAppUnits(aY/ratio) - portRect.height/2,
-                 0);
+  nsRect portRect = sf->GetScrollPortRect();
+  sf->ScrollTo(nsPoint(nsPresContext::CSSPixelsToAppUnits(aX/ratio) - portRect.width/2,
+                       nsPresContext::CSSPixelsToAppUnits(aY/ratio) - portRect.height/2),
+               nsIScrollableFrame::INSTANT);
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsImageDocument::RestoreImage()
 {
+  if (!mImageContent) {
+    return NS_OK;
+  }
   // Keep image content alive while changing the attributes.
   nsCOMPtr<nsIContent> imageContent = mImageContent;
   imageContent->UnsetAttr(kNameSpaceID_None, nsGkAtoms::width, PR_TRUE);
@@ -572,12 +548,44 @@ nsImageDocument::OnStartContainer(imgIRequest* aRequest, imgIContainer* aImage)
 {
   aImage->GetWidth(&mImageWidth);
   aImage->GetHeight(&mImageHeight);
-  CheckOverflowing(mResizeImageByDefault);
+  nsCOMPtr<nsIRunnable> runnable =
+    NS_NewRunnableMethod(this, &nsImageDocument::DefaultCheckOverflowing);
+  nsContentUtils::AddScriptRunner(runnable);
   UpdateTitleAndCharset();
 
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsImageDocument::OnStopDecode(imgIRequest *aRequest,
+                              nsresult aStatus,
+                              const PRUnichar *aStatusArg)
+{
+  UpdateTitleAndCharset();
+
+  nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(mImageContent);
+  if (imageLoader) {
+    mObservingImageLoader = PR_FALSE;
+    imageLoader->RemoveObserver(this);
+  }
+
+  // mImageContent can be null if the document is already destroyed
+  if (NS_FAILED(aStatus) && mStringBundle && mImageContent) {
+    nsCAutoString src;
+    mDocumentURI->GetSpec(src);
+    NS_ConvertUTF8toUTF16 srcString(src);
+    const PRUnichar* formatString[] = { srcString.get() };
+    nsXPIDLString errorMsg;
+    NS_NAMED_LITERAL_STRING(str, "InvalidImage");
+    mStringBundle->FormatStringFromName(str.get(), formatString, 1,
+                                        getter_Copies(errorMsg));
+
+    mImageContent->SetAttr(kNameSpaceID_None, nsGkAtoms::alt, errorMsg, PR_FALSE);
+  }
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsImageDocument::HandleEvent(nsIDOMEvent* aEvent)
 {
@@ -645,7 +653,7 @@ nsImageDocument::CreateSyntheticDocument()
   nsresult rv = nsMediaDocument::CreateSyntheticDocument();
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsIContent* body = GetBodyContent();
+  Element* body = GetBodyElement();
   if (!body) {
     NS_WARNING("no body on image document!");
     return NS_ERROR_FAILURE;
@@ -656,7 +664,7 @@ nsImageDocument::CreateSyntheticDocument()
                                            kNameSpaceID_XHTML);
   NS_ENSURE_TRUE(nodeInfo, NS_ERROR_OUT_OF_MEMORY);
 
-  mImageContent = NS_NewHTMLImageElement(nodeInfo);
+  mImageContent = NS_NewHTMLImageElement(nodeInfo.forget());
   if (!mImageContent) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -666,6 +674,12 @@ nsImageDocument::CreateSyntheticDocument()
   nsCAutoString src;
   mDocumentURI->GetSpec(src);
 
+  // Push a null JSContext on the stack so that code that runs within
+  // the below code doesn't think it's being called by JS. See bug
+  // 604262.
+  nsCxPusher pusher;
+  pusher.PushNull();
+
   NS_ConvertUTF8toUTF16 srcString(src);
   // Make sure not to start the image load from here...
   imageLoader->SetLoadingEnabled(PR_FALSE);
@@ -686,7 +700,7 @@ nsImageDocument::CheckOverflowing(PRBool changeState)
    * presentatation through style resolution is potentially dangerous.
    */
   {
-    nsIPresShell *shell = GetPrimaryShell();
+    nsIPresShell *shell = GetShell();
     if (!shell) {
       return NS_OK;
     }
@@ -694,14 +708,14 @@ nsImageDocument::CheckOverflowing(PRBool changeState)
     nsPresContext *context = shell->GetPresContext();
     nsRect visibleArea = context->GetVisibleArea();
 
-    nsIContent* content = GetBodyContent();
-    if (!content) {
+    Element* body = GetBodyElement();
+    if (!body) {
       NS_WARNING("no body on image document!");
       return NS_ERROR_FAILURE;
     }
 
     nsRefPtr<nsStyleContext> styleContext =
-      context->StyleSet()->ResolveStyleFor(content, nsnull);
+      context->StyleSet()->ResolveStyleFor(body, nsnull);
 
     nsMargin m;
     if (styleContext->GetStyleMargin()->GetMargin(m))
