@@ -55,7 +55,6 @@ class nsSVGPathElement : public nsSVGPathElementBase,
                          public nsIDOMSVGAnimatedPathData
 {
 friend class nsSVGPathFrame;
-friend class nsSVGTextPathFrame;
 
 protected:
   friend nsresult NS_NewSVGPathElement(nsIContent **aResult,
@@ -99,6 +98,8 @@ public:
     return nsGkAtoms::d;
   }
 
+  gfxFloat GetScale();
+
 protected:
 
   // nsSVGElement method
