@@ -59,10 +59,13 @@
 #include "nsGkAtoms.h"
 #include "nsXULContentUtils.h"
 #include "nsXULTemplateBuilder.h"
-#include "nsVoidArray.h"
+#include "nsIXULSortService.h"
+#include "nsTArray.h"
 #include "nsUnicharUtils.h"
 #include "nsINameSpaceManager.h"
 #include "nsIDOMClassInfo.h"
+#include "nsWhitespaceTokenizer.h"
+#include "nsTreeContentView.h"
 
 // For security check
 #include "nsIDocument.h"
@@ -87,10 +90,11 @@ public:
     // nsINativeTreeView: Untrusted code can use us
     NS_IMETHOD EnsureNative() { return NS_OK; }
 
-    virtual void NodeWillBeDestroyed(const nsINode* aNode);
+    // nsIMutationObserver
+    NS_DECL_NSIMUTATIONOBSERVER_NODEWILLBEDESTROYED
 
 protected:
-    friend NS_IMETHODIMP
+    friend nsresult
     NS_NewXULTreeBuilder(nsISupports* aOuter, REFNSIID aIID, void** aResult);
 
     nsXULTreeBuilder();
@@ -152,7 +156,7 @@ protected:
                            nsIXULTemplateResult *aResult,
                            nsTemplateQuerySet* aQuerySet,
                            PRInt32* aDelta,
-                           nsAutoVoidArray& open);
+                           nsTArray<PRInt32>& open);
 
     /**
      * Close a container row, removing the container's childrem from
@@ -262,6 +266,11 @@ protected:
      */
     Direction mSortDirection;
 
+    /*
+     * Sort hints (compare case, etc)
+     */
+    PRUint32 mSortHints;
+
     /** 
      * The builder observers.
      */
@@ -270,7 +279,7 @@ protected:
 
 //----------------------------------------------------------------------
 
-NS_IMETHODIMP
+nsresult
 NS_NewXULTreeBuilder(nsISupports* aOuter, REFNSIID aIID, void** aResult)
 {
     *aResult = nsnull;
@@ -298,15 +307,17 @@ NS_NewXULTreeBuilder(nsISupports* aOuter, REFNSIID aIID, void** aResult)
 NS_IMPL_ADDREF_INHERITED(nsXULTreeBuilder, nsXULTemplateBuilder)
 NS_IMPL_RELEASE_INHERITED(nsXULTreeBuilder, nsXULTemplateBuilder)
 
+DOMCI_DATA(XULTreeBuilder, nsXULTreeBuilder)
+
 NS_INTERFACE_MAP_BEGIN(nsXULTreeBuilder)
   NS_INTERFACE_MAP_ENTRY(nsIXULTreeBuilder)
   NS_INTERFACE_MAP_ENTRY(nsITreeView)
-  NS_INTERFACE_MAP_ENTRY_DOM_CLASSINFO(XULTreeBuilder)
+  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(XULTreeBuilder)
 NS_INTERFACE_MAP_END_INHERITING(nsXULTemplateBuilder)
 
 
 nsXULTreeBuilder::nsXULTreeBuilder()
-    : mSortDirection(eDirection_Natural)
+    : mSortDirection(eDirection_Natural), mSortHints(0)
 {
 }
 
@@ -393,6 +404,21 @@ nsXULTreeBuilder::Sort(nsIDOMElement* aElement)
     // Grab the new sort variable
     mSortVariable = do_GetAtom(sort);
 
+    nsAutoString hints;
+    header->GetAttr(kNameSpaceID_None, nsGkAtoms::sorthints, hints);
+
+    PRBool hasNaturalState = PR_TRUE;
+    nsWhitespaceTokenizer tokenizer(hints);
+    while (tokenizer.hasMoreTokens()) {
+      const nsDependentSubstring& token(tokenizer.nextToken());
+      if (token.EqualsLiteral("comparecase"))
+        mSortHints |= nsIXULSortService::SORT_COMPARECASE;
+      else if (token.EqualsLiteral("integer"))
+        mSortHints |= nsIXULSortService::SORT_INTEGER;
+      else if (token.EqualsLiteral("twostate"))
+        hasNaturalState = PR_FALSE;
+    }
+
     // Cycle the sort direction
     nsAutoString dir;
     header->GetAttr(kNameSpaceID_None, nsGkAtoms::sortDirection, dir);
@@ -401,7 +427,7 @@ nsXULTreeBuilder::Sort(nsIDOMElement* aElement)
         dir.AssignLiteral("descending");
         mSortDirection = eDirection_Descending;
     }
-    else if (dir.EqualsLiteral("descending")) {
+    else if (hasNaturalState && dir.EqualsLiteral("descending")) {
         dir.AssignLiteral("natural");
         mSortDirection = eDirection_Natural;
     }
@@ -443,6 +469,9 @@ nsXULTreeBuilder::GetSelection(nsITreeSelection** aSelection)
 NS_IMETHODIMP
 nsXULTreeBuilder::SetSelection(nsITreeSelection* aSelection)
 {
+    NS_ENSURE_TRUE(!aSelection ||
+                   nsTreeContentView::CanTrustTreeSelection(aSelection),
+                   NS_ERROR_DOM_SECURITY_ERR);
     mSelection = aSelection;
     return NS_OK;
 }
@@ -517,14 +546,12 @@ nsXULTreeBuilder::IsContainer(PRInt32 aIndex, PRBool* aResult)
 
     nsTreeRows::iterator iter = mRows[aIndex];
 
-    if (iter->mContainerType == nsTreeRows::eContainerType_Unknown) {
-        PRBool isContainer;
-        iter->mMatch->mResult->GetIsContainer(&isContainer);
+    PRBool isContainer;
+    iter->mMatch->mResult->GetIsContainer(&isContainer);
 
-        iter->mContainerType = isContainer
-            ? nsTreeRows::eContainerType_Container
-            : nsTreeRows::eContainerType_Noncontainer;
-    }
+    iter->mContainerType = isContainer
+        ? nsTreeRows::eContainerType_Container
+        : nsTreeRows::eContainerType_Noncontainer;
 
     *aResult = (iter->mContainerType == nsTreeRows::eContainerType_Container);
     return NS_OK;
@@ -1062,6 +1089,7 @@ nsXULTreeBuilder::PerformActionOnCell(const PRUnichar* aAction, PRInt32 aRow, ns
 void
 nsXULTreeBuilder::NodeWillBeDestroyed(const nsINode* aNode)
 {
+    nsCOMPtr<nsIMutationObserver> kungFuDeathGrip(this);
     if (mObservers)
         mObservers->Clear();
 
@@ -1250,7 +1278,7 @@ nsXULTreeBuilder::ReplaceMatch(nsIXULTemplateResult* aOldResult,
             if (mFlags & eDontRecurse)
                 return NS_OK;
 
-            if (result && (result != mRootResult)) {
+            if (result != mRootResult) {
                 // don't open containers if child processing isn't allowed
                 PRBool mayProcessChildren;
                 nsresult rv = result->GetMayProcessChildren(&mayProcessChildren);
@@ -1398,8 +1426,6 @@ nsXULTreeBuilder::GetTemplateActionRowFor(PRInt32 aRow, nsIContent** aResult)
     // generate text
     nsTreeRows::Row& row = *(mRows[aRow]);
 
-    nsCOMPtr<nsIContent> action;
-
     // The match stores the indices of the rule and query to use. Use these
     // to look up the right nsTemplateRule and use that rule's action to get
     // the treerow in the template.
@@ -1408,10 +1434,8 @@ nsXULTreeBuilder::GetTemplateActionRowFor(PRInt32 aRow, nsIContent** aResult)
         nsTemplateQuerySet* qs = mQuerySets[row.mMatch->QuerySetPriority()];
         nsTemplateRule* rule = qs->GetRuleAt(ruleindex);
         if (rule) {
-            rule->GetAction(getter_AddRefs(action));
-
             nsCOMPtr<nsIContent> children;
-            nsXULContentUtils::FindChildByTag(action, kNameSpaceID_XUL,
+            nsXULContentUtils::FindChildByTag(rule->GetAction(), kNameSpaceID_XUL,
                                               nsGkAtoms::treechildren,
                                               getter_AddRefs(children));
             if (children) {
@@ -1524,7 +1548,7 @@ nsXULTreeBuilder::OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
                                 nsIXULTemplateResult *aResult,
                                 PRInt32* aDelta)
 {
-    nsAutoVoidArray open;
+    nsAutoTArray<PRInt32, 8> open;
     PRInt32 count = 0;
 
     PRInt32 rulecount = mQuerySets.Length();
@@ -1536,8 +1560,8 @@ nsXULTreeBuilder::OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
 
     // Now recursively deal with any open sub-containers that just got
     // inserted. We need to do this back-to-front to avoid skewing offsets.
-    for (PRInt32 i = open.Count() - 1; i >= 0; --i) {
-        PRInt32 index = NS_PTR_TO_INT32(open[i]);
+    for (PRInt32 i = open.Length() - 1; i >= 0; --i) {
+        PRInt32 index = open[i];
 
         nsTreeRows::Subtree* child =
             mRows.EnsureSubtreeFor(aSubtree, index);
@@ -1568,7 +1592,7 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
                                          nsIXULTemplateResult* aResult,
                                          nsTemplateQuerySet* aQuerySet,
                                          PRInt32* aDelta,
-                                         nsAutoVoidArray& open)
+                                         nsTArray<PRInt32>& open)
 {
     PRInt32 count = *aDelta;
     
@@ -1676,12 +1700,16 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
                 PRBool isOpen = PR_FALSE;
                 IsContainerOpen(nextresult, &isOpen);
                 if (isOpen) {
-                    if (!open.AppendElement(NS_INT32_TO_PTR(count)))
+                    if (open.AppendElement(count) == nsnull)
                         return NS_ERROR_OUT_OF_MEMORY;
                 }
 
                 ++count;
             }
+
+            if (mFlags & eLoggingEnabled)
+                OutputMatchToLog(resultid, newmatch, PR_TRUE);
+
         }
 
         if (prevmatch) {
@@ -1706,12 +1734,9 @@ nsXULTreeBuilder::CloseContainer(PRInt32 aIndex)
 
     nsTreeRows::iterator iter = mRows[aIndex];
 
-    nsTreeRows::Subtree& subtree = *(iter->mSubtree);
+    if (iter->mSubtree)
+        RemoveMatchesFor(*iter->mSubtree);
 
-    RemoveMatchesFor(subtree);
-
-    // Update the view
-    iter = mRows[aIndex];
 
     PRInt32 count = mRows.GetSubtreeSizeFor(iter);
     mRows.RemoveSubtreeFor(iter);
@@ -1866,7 +1891,7 @@ nsXULTreeBuilder::CompareResults(nsIXULTemplateResult* aLeft, nsIXULTemplateResu
     }
 
     PRInt32 sortorder;
-    mQueryProcessor->CompareResults(aLeft, aRight, mSortVariable, &sortorder);
+    mQueryProcessor->CompareResults(aLeft, aRight, mSortVariable, mSortHints, &sortorder);
 
     if (sortorder)
         sortorder = sortorder * mSortDirection;
@@ -1894,7 +1919,8 @@ nsXULTreeBuilder::SortSubtree(nsTreeRows::Subtree* aSubtree)
 
 /* boolean canDrop (in long index, in long orientation); */
 NS_IMETHODIMP
-nsXULTreeBuilder::CanDrop(PRInt32 index, PRInt32 orientation, PRBool *_retval)
+nsXULTreeBuilder::CanDrop(PRInt32 index, PRInt32 orientation,
+                          nsIDOMDataTransfer* dataTransfer, PRBool *_retval)
 {
     *_retval = PR_FALSE;
     if (mObservers) {
@@ -1903,7 +1929,7 @@ nsXULTreeBuilder::CanDrop(PRInt32 index, PRInt32 orientation, PRBool *_retval)
         for (PRUint32 i = 0; i < count; ++i) {
             nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
             if (observer) {
-                observer->CanDrop(index, orientation, _retval);
+                observer->CanDrop(index, orientation, dataTransfer, _retval);
                 if (*_retval)
                     break;
             }
@@ -1914,7 +1940,7 @@ nsXULTreeBuilder::CanDrop(PRInt32 index, PRInt32 orientation, PRBool *_retval)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::Drop(PRInt32 row, PRInt32 orient)
+nsXULTreeBuilder::Drop(PRInt32 row, PRInt32 orient, nsIDOMDataTransfer* dataTransfer)
 {
     if (mObservers) {
         PRUint32 count;
@@ -1923,9 +1949,9 @@ nsXULTreeBuilder::Drop(PRInt32 row, PRInt32 orient)
             nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
             if (observer) {
                 PRBool canDrop = PR_FALSE;
-                observer->CanDrop(row, orient, &canDrop);
+                observer->CanDrop(row, orient, dataTransfer, &canDrop);
                 if (canDrop)
-                    observer->OnDrop(row, orient);
+                    observer->OnDrop(row, orient, dataTransfer);
             }
         }
     }
