@@ -71,7 +71,6 @@
 #include "nsIDOMNSRange.h"
 #include "nsEditorUtils.h"
 #include "nsIDOMEventTarget.h"
-#include "nsIEventStateManager.h"
 #include "nsISelectionPrivate.h"
 #include "nsIDOMDragEvent.h"
 #include "nsIFocusManager.h"
@@ -258,9 +257,7 @@ nsEditorEventListener::GetPresShell()
 {
   NS_PRECONDITION(mEditor,
     "The caller must check whether this is connected to an editor");
-  nsCOMPtr<nsIPresShell> ps;
-  mEditor->GetPresShell(getter_AddRefs(ps));
-  return ps.forget();
+  return mEditor->GetPresShell();
 }
 
 /**
@@ -553,14 +550,10 @@ nsEditorEventListener::DragEnter(nsIDOMDragEvent* aDragEvent)
   nsCOMPtr<nsIPresShell> presShell = GetPresShell();
   NS_ENSURE_TRUE(presShell, NS_OK);
 
-  if (!mCaret)
-  {
-    NS_NewCaret(getter_AddRefs(mCaret));
-    if (mCaret)
-    {
-      mCaret->Init(presShell);
-      mCaret->SetCaretReadOnly(PR_TRUE);
-    }
+  if (!mCaret) {
+    mCaret = new nsCaret();
+    mCaret->Init(presShell);
+    mCaret->SetCaretReadOnly(PR_TRUE);
   }
 
   presShell->SetCaret(mCaret);
