@@ -1,6 +1,5 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- *
- * ***** BEGIN LICENSE BLOCK *****
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
  * The contents of this file are subject to the Mozilla Public License Version
@@ -1028,11 +1027,6 @@ nsDocShellTreeOwner::GetOwnerRequestor()
 }
 
 
-#ifdef XP_MAC
-#pragma mark -
-#endif
-
-
 ///////////////////////////////////////////////////////////////////////////////
 // DefaultTooltipTextProvider
 
@@ -1570,8 +1564,7 @@ ChromeTooltipListener::sTooltipCallback(nsITimer *aTimer,
     if (shell) {
       nsIViewManager* vm = shell->GetViewManager();
       if (vm) {
-        nsIView* view;
-        vm->GetRootView(view);
+        nsIView* view = vm->GetRootView();
         if (view) {
           nsPoint offset;
           widget = view->GetNearestWidget(&offset);
@@ -1653,12 +1646,6 @@ ChromeTooltipListener::sAutoHideCallback(nsITimer *aTimer, void* aListener)
 } // sAutoHideCallback
 
 
-
-#ifdef XP_MAC
-#pragma mark -
-#endif
-
-
 NS_IMPL_ADDREF(ChromeContextMenuListener)
 NS_IMPL_RELEASE(ChromeContextMenuListener)
 
