@@ -38,33 +38,39 @@
 
 
 /**
-   File Name:          7.4.3-12-n.js
-   ECMA Section:       7.4.3
+   File Name:          7.4.2-15-n.js
+   ECMA Section:       7.4.2
 
    Description:
-   The following words are used as keywords in proposed extensions and are
-   therefore reserved to allow for the possibility of future adoption of
-   those extensions.
+   The following tokens are ECMAScript keywords and may not be used as
+   identifiers in ECMAScript programs.
 
-   FutureReservedWord :: one of
-   case    debugger    export      super
-   catch   default     extends     switch
-   class   do          finally     throw
-   const   enum        import      try
+   Syntax
+
+   Keyword :: one of
+   break          for         new         var
+   continue       function    return      void
+   delete         if          this        while
+   else           in          typeof      with
+
+   This test verifies that the keyword cannot be used as an identifier.
+   Functioinal tests of the keyword may be found in the section corresponding
+   to the function of the keyword.
 
    Author:             christine@netscape.com
    Date:               12 november 1997
+
 */
-var SECTION = "7.4.3-12-n";
+var SECTION = "7.4.1-15-n";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Future Reserved Words";
+var TITLE   = "Keywords";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-DESCRIPTION = "var throw = true";
+DESCRIPTION = "var typeof = true";
 EXPECTED = "error";
 
-new TestCase( SECTION,  "var throw = true",     "error",    eval("var throw = true") );
+new TestCase( SECTION,  "var typeof = true",     "error",    eval("var typeof = true") );
 
 test();
