@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s):
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,12 +36,11 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 360969;
-var summary = '2^17: local const';
-var actual = 'No Crash';
-var expect = 'No Crash';
+var BUGNUMBER = 355829;
+var summary = 'js_ValueToObject should return the original object if OBJ_DEFAULT_VALUE returns a primitive value';
+var actual = '';
+var expect = '';
 
-var global = this;
 
 //-----------------------------------------------------------------------------
 test();
@@ -52,22 +51,23 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  var start = new Date();
-  var p;
-  var i;
-  var limit = 2 << 16;
 
-  for (var i = 0; i < limit; i++)
-  {
-    eval('const pv' + i + ';');
+  expect = actual = 'Good conversion';
+
+  var primitiveValues = [
+    true, false, 0, 1, -2, 0.1, -2e100, 0/0, 1/0, -1/1,  "", "xxx",
+    undefined, null
+    ];
+
+  for (var i = 0; i != primitiveValues.length; ++i) {
+    var v = primitiveValues[i];
+    var obj = { valueOf: function() { return v; } };
+    var obj2 = Object(obj);
+    if (obj !== obj2)
+      actual = "Bad conversion for '"+v + "'";
   }
 
   reportCompare(expect, actual, summary);
 
-  var stop = new Date();
-
-  print('Elapsed time: ' + Math.floor((stop - start)/1000) + ' seconds');
-
   exitFunc ('test');
 }
