@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Gary Kwong
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,8 +36,8 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 470388;
-var summary = 'TM: Do not assert: !(fp->flags & JSFRAME_POP_BLOCKS)';
+var BUGNUMBER = 465460;
+var summary = 'TM: valueOf in a loop: do not assert';
 var actual = '';
 var expect = '';
 
@@ -54,7 +54,7 @@ function test()
  
   jit(true);
 
-  for each (let x in [function(){}, {}, {}, function(){}, function(){}]) { ([<y/> for (y in x)]); }
+  for (let d=0;d<2;++d) for (let a=0;a<1;++a) "" + (d && function(){});
 
   jit(false);
 
