@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Mats Palmgren <mats.palmgren@bredband.net>
+ *   Jonathon Jongsma <jonathon.jongsma@collabora.co.uk>, Collabora Ltd.
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -41,6 +42,7 @@
 #ifndef nsStyleConsts_h___
 #define nsStyleConsts_h___
 
+#include "gfxRect.h"
 #include "nsFont.h"
 
 // cairo doesn't support invert
@@ -49,18 +51,18 @@
 // XXX fold this into nsStyleContext and group by nsStyleXXX struct
 
 // Indices into border/padding/margin arrays
-#define NS_SIDE_TOP     0
-#define NS_SIDE_RIGHT   1
-#define NS_SIDE_BOTTOM  2
-#define NS_SIDE_LEFT    3
+#define NS_SIDE_TOP     mozilla::css::eSideTop
+#define NS_SIDE_RIGHT   mozilla::css::eSideRight
+#define NS_SIDE_BOTTOM  mozilla::css::eSideBottom
+#define NS_SIDE_LEFT    mozilla::css::eSideLeft
 
-#define NS_FOR_CSS_SIDES(var_) for (PRInt32 var_ = 0; var_ < 4; ++var_)
-
-// Indices into "full corner" arrays (nsCSSCornerSizes e.g.)
-#define NS_CORNER_TOP_LEFT     0
-#define NS_CORNER_TOP_RIGHT    1
-#define NS_CORNER_BOTTOM_RIGHT 2
-#define NS_CORNER_BOTTOM_LEFT  3
+#define NS_FOR_CSS_SIDES(var_) for (mozilla::css::Side var_ = NS_SIDE_TOP; var_ <= NS_SIDE_LEFT; var_++)
+static inline mozilla::css::Side operator++(mozilla::css::Side& side, int) {
+    NS_PRECONDITION(side >= NS_SIDE_TOP &&
+                    side <= NS_SIDE_LEFT, "Out of range side");
+    side = mozilla::css::Side(side + 1);
+    return side;
+}
 
 #define NS_FOR_CSS_FULL_CORNERS(var_) for (PRInt32 var_ = 0; var_ < 4; ++var_)
 
@@ -160,6 +162,10 @@
 #define NS_STYLE_BOX_ORIENT_HORIZONTAL 0
 #define NS_STYLE_BOX_ORIENT_VERTICAL   1
 
+// orient
+#define NS_STYLE_ORIENT_HORIZONTAL 0
+#define NS_STYLE_ORIENT_VERTICAL   1
+
 // stack-sizing
 #define NS_STYLE_STACK_SIZING_IGNORE         0
 #define NS_STYLE_STACK_SIZING_STRETCH_TO_FIT 1
@@ -197,19 +203,19 @@
 #define NS_STYLE_SPEAK_NORMAL             1
 #define NS_STYLE_SPEAK_SPELL_OUT          2
 
-// See nsStyleAural 
+// See nsStyleAural
 #define NS_STYLE_SPEAK_HEADER_ONCE        0
 #define NS_STYLE_SPEAK_HEADER_ALWAYS      1
 
-// See nsStyleAural 
+// See nsStyleAural
 #define NS_STYLE_SPEAK_NUMERAL_DIGITS     0
 #define NS_STYLE_SPEAK_NUMERAL_CONTINUOUS 1
 
-// See nsStyleAural 
+// See nsStyleAural
 #define NS_STYLE_SPEAK_PUNCTUATION_NONE   0
 #define NS_STYLE_SPEAK_PUNCTUATION_CODE   1
 
-// See nsStyleAural 
+// See nsStyleAural
 #define NS_STYLE_SPEECH_RATE_X_SLOW       0
 #define NS_STYLE_SPEECH_RATE_SLOW         1
 #define NS_STYLE_SPEECH_RATE_MEDIUM       2
@@ -218,7 +224,7 @@
 #define NS_STYLE_SPEECH_RATE_FASTER       10
 #define NS_STYLE_SPEECH_RATE_SLOWER       11
 
-// See nsStyleAural 
+// See nsStyleAural
 #define NS_STYLE_VOLUME_SILENT            0
 #define NS_STYLE_VOLUME_X_SOFT            1
 #define NS_STYLE_VOLUME_SOFT              2
@@ -233,10 +239,31 @@
 #endif
 
 // See nsStyleColor
-#define NS_COLOR_MOZ_HYPERLINKTEXT              -1
-#define NS_COLOR_MOZ_VISITEDHYPERLINKTEXT       -2
-#define NS_COLOR_MOZ_ACTIVEHYPERLINKTEXT        -3
-#define NS_COLOR_CURRENTCOLOR                   -4
+#define NS_COLOR_CURRENTCOLOR                   -1
+#define NS_COLOR_MOZ_DEFAULT_COLOR              -2
+#define NS_COLOR_MOZ_DEFAULT_BACKGROUND_COLOR   -3
+#define NS_COLOR_MOZ_HYPERLINKTEXT              -4
+#define NS_COLOR_MOZ_VISITEDHYPERLINKTEXT       -5
+#define NS_COLOR_MOZ_ACTIVEHYPERLINKTEXT        -6
+
+#ifdef MOZ_CSS_ANIMATIONS
+// See nsStyleDisplay
+#define NS_STYLE_ANIMATION_DIRECTION_NORMAL       0
+#define NS_STYLE_ANIMATION_DIRECTION_ALTERNATE    1
+
+// See nsStyleDisplay
+#define NS_STYLE_ANIMATION_FILL_MODE_NONE         0
+#define NS_STYLE_ANIMATION_FILL_MODE_FORWARDS     1
+#define NS_STYLE_ANIMATION_FILL_MODE_BACKWARDS    2
+#define NS_STYLE_ANIMATION_FILL_MODE_BOTH         3
+
+// See nsStyleDisplay
+#define NS_STYLE_ANIMATION_ITERATION_COUNT_INFINITE 0
+
+// See nsStyleDisplay
+#define NS_STYLE_ANIMATION_PLAY_STATE_RUNNING     0
+#define NS_STYLE_ANIMATION_PLAY_STATE_PAUSED      1
+#endif
 
 // See nsStyleBackground
 #define NS_STYLE_BG_ATTACHMENT_SCROLL     0
@@ -246,8 +273,7 @@
 // Code depends on these constants having the same values as BG_ORIGIN_*
 #define NS_STYLE_BG_CLIP_BORDER           0
 #define NS_STYLE_BG_CLIP_PADDING          1
-// When we add NS_STYLE_BG_CLIP_CONTENT, we should add the PR_STATIC_ASSERTs
-// to the places that assert equality for BORDER and PADDING.
+#define NS_STYLE_BG_CLIP_CONTENT          2
 
 // See nsStyleBackground
 #define NS_STYLE_BG_INLINE_POLICY_EACH_BOX      0
@@ -301,9 +327,6 @@
 #define NS_STYLE_BORDER_STYLE_OUTSET            8
 #define NS_STYLE_BORDER_STYLE_HIDDEN            9
 #define NS_STYLE_BORDER_STYLE_AUTO              10 // for outline-style only
-// a bit ORed onto the style for table border collapsing indicating that the style was 
-// derived from a table with its rules attribute set
-#define NS_STYLE_BORDER_STYLE_RULES_MARKER      0x10  
 
 // See nsStyleBorder mBorderImage
 #define NS_STYLE_BORDER_IMAGE_STRETCH           0
@@ -333,15 +356,15 @@
 #define NS_STYLE_CURSOR_CROSSHAIR               2
 #define NS_STYLE_CURSOR_DEFAULT                 3    // ie: an arrow
 #define NS_STYLE_CURSOR_POINTER                 4    // for links
-#define NS_STYLE_CURSOR_MOVE                    5 
-#define NS_STYLE_CURSOR_E_RESIZE                6     
-#define NS_STYLE_CURSOR_NE_RESIZE               7      
-#define NS_STYLE_CURSOR_NW_RESIZE               8      
-#define NS_STYLE_CURSOR_N_RESIZE                9     
-#define NS_STYLE_CURSOR_SE_RESIZE               10      
-#define NS_STYLE_CURSOR_SW_RESIZE               11     
-#define NS_STYLE_CURSOR_S_RESIZE                12    
-#define NS_STYLE_CURSOR_W_RESIZE                13    
+#define NS_STYLE_CURSOR_MOVE                    5
+#define NS_STYLE_CURSOR_E_RESIZE                6
+#define NS_STYLE_CURSOR_NE_RESIZE               7
+#define NS_STYLE_CURSOR_NW_RESIZE               8
+#define NS_STYLE_CURSOR_N_RESIZE                9
+#define NS_STYLE_CURSOR_SE_RESIZE               10
+#define NS_STYLE_CURSOR_SW_RESIZE               11
+#define NS_STYLE_CURSOR_S_RESIZE                12
+#define NS_STYLE_CURSOR_W_RESIZE                13
 #define NS_STYLE_CURSOR_TEXT                    14   // ie: i-beam
 #define NS_STYLE_CURSOR_WAIT                    15
 #define NS_STYLE_CURSOR_HELP                    16
@@ -377,9 +400,6 @@
 #define NS_STYLE_DISPLAY_INLINE                 2
 #define NS_STYLE_DISPLAY_INLINE_BLOCK           3
 #define NS_STYLE_DISPLAY_LIST_ITEM              4
-#define NS_STYLE_DISPLAY_MARKER                 5
-#define NS_STYLE_DISPLAY_RUN_IN                 6
-#define NS_STYLE_DISPLAY_COMPACT                7
 #define NS_STYLE_DISPLAY_TABLE                  8
 #define NS_STYLE_DISPLAY_INLINE_TABLE           9
 #define NS_STYLE_DISPLAY_TABLE_ROW_GROUP        10
@@ -424,8 +444,9 @@
 // We should eventually stop using the NS_STYLE_* variants here.
 #define NS_STYLE_FONT_WEIGHT_NORMAL             NS_FONT_WEIGHT_NORMAL
 #define NS_STYLE_FONT_WEIGHT_BOLD               NS_FONT_WEIGHT_BOLD
-#define NS_STYLE_FONT_WEIGHT_BOLDER             NS_FONT_WEIGHT_BOLDER
-#define NS_STYLE_FONT_WEIGHT_LIGHTER            NS_FONT_WEIGHT_LIGHTER
+// The constants below appear only in style sheets and not computed style.
+#define NS_STYLE_FONT_WEIGHT_BOLDER             (-1)
+#define NS_STYLE_FONT_WEIGHT_LIGHTER            (-2)
 
 // See nsStyleFont
 #define NS_STYLE_FONT_SIZE_XXSMALL              0
@@ -450,8 +471,6 @@
 #define NS_STYLE_FONT_STRETCH_EXPANDED          NS_FONT_STRETCH_EXPANDED
 #define NS_STYLE_FONT_STRETCH_EXTRA_EXPANDED    NS_FONT_STRETCH_EXTRA_EXPANDED
 #define NS_STYLE_FONT_STRETCH_ULTRA_EXPANDED    NS_FONT_STRETCH_ULTRA_EXPANDED
-#define NS_STYLE_FONT_STRETCH_WIDER             NS_FONT_STRETCH_WIDER
-#define NS_STYLE_FONT_STRETCH_NARROWER          NS_FONT_STRETCH_NARROWER
 
 // See nsStyleFont - system fonts
 #define NS_STYLE_FONT_CAPTION                   1		// css2
@@ -497,7 +516,7 @@
 #define NS_STYLE_CLIP_BOTTOM_AUTO               0x80
 
 // FRAME/FRAMESET/IFRAME specific values including backward compatibility. Boolean values with
-// the same meaning (e.g. 1 & yes) may need to be distinguished for correct mode processing 
+// the same meaning (e.g. 1 & yes) may need to be distinguished for correct mode processing
 #define NS_STYLE_FRAME_YES                      0
 #define NS_STYLE_FRAME_NO                       1
 #define NS_STYLE_FRAME_0                        2
@@ -554,7 +573,7 @@
 #define NS_STYLE_LIST_STYLE_MOZ_JAPANESE_FORMAL       31
 #define NS_STYLE_LIST_STYLE_MOZ_ARABIC_INDIC          32
 #define NS_STYLE_LIST_STYLE_MOZ_PERSIAN               33
-#define NS_STYLE_LIST_STYLE_MOZ_URDU                  34 
+#define NS_STYLE_LIST_STYLE_MOZ_URDU                  34
 #define NS_STYLE_LIST_STYLE_MOZ_DEVANAGARI            35
 #define NS_STYLE_LIST_STYLE_MOZ_GURMUKHI              36
 #define NS_STYLE_LIST_STYLE_MOZ_GUJARATI              37
@@ -595,6 +614,12 @@
 #define NS_STYLE_POINTER_EVENTS_ALL             8
 #define NS_STYLE_POINTER_EVENTS_AUTO            9
 
+// See nsStyleDisplay
+#define NS_STYLE_RESIZE_NONE                    0
+#define NS_STYLE_RESIZE_BOTH                    1
+#define NS_STYLE_RESIZE_HORIZONTAL              2
+#define NS_STYLE_RESIZE_VERTICAL                3
+
 // See nsStyleText
 #define NS_STYLE_TEXT_ALIGN_DEFAULT               0
 #define NS_STYLE_TEXT_ALIGN_LEFT                  1
@@ -612,17 +637,29 @@
 // Note: make sure that the largest NS_STYLE_TEXT_ALIGN_* value is smaller than
 // the smallest NS_STYLE_VERTICAL_ALIGN_* value below!
 
+// See nsStyleText
+#define NS_STYLE_TEXT_BLINK_NONE                0
+#define NS_STYLE_TEXT_BLINK_BLINK               1
+
 // See nsStyleText, nsStyleFont
-#define NS_STYLE_TEXT_DECORATION_NONE           0
-#define NS_STYLE_TEXT_DECORATION_UNDERLINE      NS_FONT_DECORATION_UNDERLINE
-#define NS_STYLE_TEXT_DECORATION_OVERLINE       NS_FONT_DECORATION_OVERLINE
-#define NS_STYLE_TEXT_DECORATION_LINE_THROUGH   NS_FONT_DECORATION_LINE_THROUGH
-#define NS_STYLE_TEXT_DECORATION_BLINK          0x08
-#define NS_STYLE_TEXT_DECORATION_PREF_ANCHORS   0x10
+#define NS_STYLE_TEXT_DECORATION_LINE_NONE         0
+#define NS_STYLE_TEXT_DECORATION_LINE_UNDERLINE    NS_FONT_DECORATION_UNDERLINE
+#define NS_STYLE_TEXT_DECORATION_LINE_OVERLINE     NS_FONT_DECORATION_OVERLINE
+#define NS_STYLE_TEXT_DECORATION_LINE_LINE_THROUGH NS_FONT_DECORATION_LINE_THROUGH
+#define NS_STYLE_TEXT_DECORATION_LINE_PREF_ANCHORS 0x10
 // OVERRIDE_ALL does not occur in stylesheets; it only comes from HTML
 // attribute mapping (and thus appears in computed data)
-#define NS_STYLE_TEXT_DECORATION_OVERRIDE_ALL   0x20
-#define NS_STYLE_TEXT_DECORATION_LINES_MASK     (NS_STYLE_TEXT_DECORATION_UNDERLINE | NS_STYLE_TEXT_DECORATION_OVERLINE | NS_STYLE_TEXT_DECORATION_LINE_THROUGH)
+#define NS_STYLE_TEXT_DECORATION_LINE_OVERRIDE_ALL 0x20
+#define NS_STYLE_TEXT_DECORATION_LINE_LINES_MASK   (NS_STYLE_TEXT_DECORATION_LINE_UNDERLINE | NS_STYLE_TEXT_DECORATION_LINE_OVERLINE | NS_STYLE_TEXT_DECORATION_LINE_LINE_THROUGH)
+
+// See nsStyleText
+#define NS_STYLE_TEXT_DECORATION_STYLE_NONE     0 // not in CSS spec, mapped to -moz-none
+#define NS_STYLE_TEXT_DECORATION_STYLE_DOTTED   1
+#define NS_STYLE_TEXT_DECORATION_STYLE_DASHED   2
+#define NS_STYLE_TEXT_DECORATION_STYLE_SOLID    3
+#define NS_STYLE_TEXT_DECORATION_STYLE_DOUBLE   4
+#define NS_STYLE_TEXT_DECORATION_STYLE_WAVY     5
+#define NS_STYLE_TEXT_DECORATION_STYLE_MAX      NS_STYLE_TEXT_DECORATION_STYLE_WAVY
 
 // See nsStyleText
 #define NS_STYLE_TEXT_TRANSFORM_NONE            0
@@ -630,6 +667,15 @@
 #define NS_STYLE_TEXT_TRANSFORM_LOWERCASE       2
 #define NS_STYLE_TEXT_TRANSFORM_UPPERCASE       3
 
+// See nsStyleDisplay
+#define NS_STYLE_TRANSITION_TIMING_FUNCTION_EASE         0
+#define NS_STYLE_TRANSITION_TIMING_FUNCTION_LINEAR       1
+#define NS_STYLE_TRANSITION_TIMING_FUNCTION_EASE_IN      2
+#define NS_STYLE_TRANSITION_TIMING_FUNCTION_EASE_OUT     3
+#define NS_STYLE_TRANSITION_TIMING_FUNCTION_EASE_IN_OUT  4
+#define NS_STYLE_TRANSITION_TIMING_FUNCTION_STEP_START   5
+#define NS_STYLE_TRANSITION_TIMING_FUNCTION_STEP_END     6
+
 // See nsStyleText
 // Note: these values pickup after the text-align values because there
 // are a few html cases where an object can have both types of
@@ -649,6 +695,9 @@
 #define NS_STYLE_VISIBILITY_VISIBLE             1
 #define NS_STYLE_VISIBILITY_COLLAPSE            2
 
+// See nsStyleText
+#define NS_STYLE_TABSIZE_INITIAL                8
+
 // See nsStyleText
 #define NS_STYLE_WHITESPACE_NORMAL              0
 #define NS_STYLE_WHITESPACE_PRE                 1
@@ -660,6 +709,11 @@
 #define NS_STYLE_WORDWRAP_NORMAL                0
 #define NS_STYLE_WORDWRAP_BREAK_WORD            1
 
+// See nsStyleText
+#define NS_STYLE_HYPHENS_NONE                   0
+#define NS_STYLE_HYPHENS_MANUAL                 1
+#define NS_STYLE_HYPHENS_AUTO                   2
+
 // See nsStyleText
 #define NS_STYLE_LINE_HEIGHT_BLOCK_HEIGHT       0
 
@@ -747,7 +801,6 @@
 #define NS_STYLE_GRADIENT_SIZE_FARTHEST_SIDE    2
 #define NS_STYLE_GRADIENT_SIZE_FARTHEST_CORNER  3
 
-#ifdef MOZ_SVG
 // See nsStyleSVG
 
 // dominant-baseline
@@ -791,9 +844,9 @@
 #define NS_STYLE_STROKE_LINEJOIN_BEVEL          2
 
 // text-anchor
-#define NS_STYLE_TEXT_ANCHOR_START              0 
-#define NS_STYLE_TEXT_ANCHOR_MIDDLE             1 
-#define NS_STYLE_TEXT_ANCHOR_END                2 
+#define NS_STYLE_TEXT_ANCHOR_START              0
+#define NS_STYLE_TEXT_ANCHOR_MIDDLE             1
+#define NS_STYLE_TEXT_ANCHOR_END                2
 
 // text-rendering
 #define NS_STYLE_TEXT_RENDERING_AUTO               0
@@ -806,8 +859,6 @@
 #define NS_STYLE_COLOR_INTERPOLATION_SRGB           1
 #define NS_STYLE_COLOR_INTERPOLATION_LINEARRGB      2
 
-#endif // MOZ_SVG
-
 /*****************************************************************************
  * Constants for media features.                                             *
  *****************************************************************************/
