@@ -41,15 +41,15 @@
 #include "nsTreeUtils.h"
 #include "nsTreeContentView.h"
 #include "nsChildIterator.h"
-#include "nsIDOMHTMLOptionElement.h"
-#include "nsIDOMHTMLOptGroupElement.h"
 #include "nsIDOMClassInfo.h"
-#include "nsIEventStateManager.h"
+#include "nsEventStates.h"
 #include "nsINodeInfo.h"
 #include "nsIXULSortService.h"
 #include "nsContentUtils.h"
 #include "nsTreeBodyFrame.h"
-#include "nsDOMError.h"
+#include "mozilla/dom/Element.h"
+
+namespace dom = mozilla::dom;
 
 #define NS_ENSURE_NATIVE_COLUMN(_col)                                \
   nsRefPtr<nsTreeColumn> col = nsTreeBodyFrame::GetColumnImpl(_col); \
@@ -138,8 +138,7 @@ nsTreeContentView::nsTreeContentView(void) :
   mBoxObject(nsnull),
   mSelection(nsnull),
   mRoot(nsnull),
-  mDocument(nsnull),
-  mUpdateSelection(PR_FALSE)
+  mDocument(nsnull)
 {
   static const size_t kBucketSizes[] = {
     sizeof(Row)
@@ -167,10 +166,18 @@ NS_NewTreeContentView(nsITreeView** aResult)
   return NS_OK;
 }
 
-NS_IMPL_ADDREF(nsTreeContentView)
-NS_IMPL_RELEASE(nsTreeContentView)
+NS_IMPL_CYCLE_COLLECTION_4(nsTreeContentView,
+                           mBoxObject,
+                           mSelection,
+                           mRoot,
+                           mBody)
 
-NS_INTERFACE_MAP_BEGIN(nsTreeContentView)
+NS_IMPL_CYCLE_COLLECTING_ADDREF(nsTreeContentView)
+NS_IMPL_CYCLE_COLLECTING_RELEASE(nsTreeContentView)
+
+DOMCI_DATA(TreeContentView, nsTreeContentView)
+
+NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsTreeContentView)
   NS_INTERFACE_MAP_ENTRY(nsITreeView)
   NS_INTERFACE_MAP_ENTRY(nsITreeContentView)
   NS_INTERFACE_MAP_ENTRY(nsIDocumentObserver)
@@ -212,24 +219,6 @@ nsTreeContentView::SetSelection(nsITreeSelection* aSelection)
                  NS_ERROR_DOM_SECURITY_ERR);
 
   mSelection = aSelection;
-  if (!mSelection || !mUpdateSelection)
-    return NS_OK;
-
-  mUpdateSelection = PR_FALSE;
-
-  mSelection->SetSelectEventsSuppressed(PR_TRUE);
-  for (PRUint32 i = 0; i < mRows.Length(); ++i) {
-    Row* row = mRows[i];
-    nsCOMPtr<nsIDOMHTMLOptionElement> optEl = do_QueryInterface(row->mContent);
-    if (optEl) {
-      PRBool isSelected;
-      optEl->GetSelected(&isSelected);
-      if (isSelected)
-        mSelection->ToggleSelect(i);
-    }
-  }
-  mSelection->SetSelectEventsSuppressed(PR_FALSE);
-
   return NS_OK;
 }
 
@@ -526,25 +515,13 @@ nsTreeContentView::GetCellText(PRInt32 aRow, nsITreeColumn* aCol, nsAString& _re
   Row* row = mRows[aRow];
 
   // Check for a "label" attribute - this is valid on an <treeitem>
-  // or an <option>, with a single implied column.
+  // with a single implied column.
   if (row->mContent->GetAttr(kNameSpaceID_None, nsGkAtoms::label, _retval)
       && !_retval.IsEmpty())
     return NS_OK;
 
   nsIAtom *rowTag = row->mContent->Tag();
-  if (rowTag == nsGkAtoms::option &&
-      row->mContent->IsNodeOfType(nsINode::eHTML)) {
-    // Use the text node child as the label
-    nsCOMPtr<nsIDOMHTMLOptionElement> elem = do_QueryInterface(row->mContent);
-    elem->GetText(_retval);
-  }
-  else if (rowTag == nsGkAtoms::optgroup &&
-           row->mContent->IsNodeOfType(nsINode::eHTML)) {
-    nsCOMPtr<nsIDOMHTMLOptGroupElement> elem = do_QueryInterface(row->mContent);
-    elem->GetLabel(_retval);
-  }
-  else if (rowTag == nsGkAtoms::treeitem &&
-           row->mContent->IsNodeOfType(nsINode::eXUL)) {
+  if (rowTag == nsGkAtoms::treeitem && row->mContent->IsXUL()) {
     nsIContent* realRow =
       nsTreeUtils::GetImmediateChild(row->mContent, nsGkAtoms::treerow);
     if (realRow) {
@@ -571,6 +548,7 @@ nsTreeContentView::SetTree(nsITreeBoxObject* aTree)
     boxObject->GetElement(getter_AddRefs(element));
 
     mRoot = do_QueryInterface(element);
+    NS_ENSURE_STATE(mRoot);
 
     // Add ourselves to document's observers.
     nsIDocument* document = mRoot->GetDocument();
@@ -602,20 +580,10 @@ nsTreeContentView::ToggleOpenState(PRInt32 aIndex)
   // lazily.
   Row* row = mRows[aIndex];
 
-  if (row->mContent->Tag() == nsGkAtoms::optgroup &&
-      row->mContent->IsNodeOfType(nsINode::eHTML)) {
-    // we don't use an attribute for optgroup's open state
-    if (row->IsOpen())
-      CloseContainer(aIndex);
-    else
-      OpenContainer(aIndex);
-  }
-  else {
-    if (row->IsOpen())
-      row->mContent->SetAttr(kNameSpaceID_None, nsGkAtoms::open, NS_LITERAL_STRING("false"), PR_TRUE);
-    else
-      row->mContent->SetAttr(kNameSpaceID_None, nsGkAtoms::open, NS_LITERAL_STRING("true"), PR_TRUE);
-  }
+  if (row->IsOpen())
+    row->mContent->SetAttr(kNameSpaceID_None, nsGkAtoms::open, NS_LITERAL_STRING("false"), PR_TRUE);
+  else
+    row->mContent->SetAttr(kNameSpaceID_None, nsGkAtoms::open, NS_LITERAL_STRING("true"), PR_TRUE);
 
   return NS_OK;
 }
@@ -648,6 +616,11 @@ nsTreeContentView::CycleHeader(nsITreeColumn* aCol)
           default: sortdirection.AssignLiteral("ascending"); break;
         }
 
+        nsAutoString hints;
+        column->GetAttr(kNameSpaceID_None, nsGkAtoms::sorthints, hints);
+        sortdirection.AppendLiteral(" ");
+        sortdirection += hints;
+
         nsCOMPtr<nsIDOMNode> rootnode = do_QueryInterface(mRoot);
         xs->Sort(rootnode, sort, sortdirection);
       }
@@ -804,76 +777,57 @@ nsTreeContentView::GetIndexOfItem(nsIDOMElement* aItem, PRInt32* _retval)
 }
 
 void
-nsTreeContentView::ContentStatesChanged(nsIDocument* aDocument,
-                                        nsIContent* aContent1,
-                                        nsIContent* aContent2,
-                                        PRInt32 aStateMask)
+nsTreeContentView::AttributeChanged(nsIDocument*  aDocument,
+                                    dom::Element* aElement,
+                                    PRInt32       aNameSpaceID,
+                                    nsIAtom*      aAttribute,
+                                    PRInt32       aModType)
 {
-  if (!aContent1 || !mSelection ||
-      !aContent1->IsNodeOfType(nsINode::eHTML) ||
-      !(aStateMask & NS_EVENT_STATE_CHECKED))
-    return;
+  // Lots of codepaths under here that do all sorts of stuff, so be safe.
+  nsCOMPtr<nsIMutationObserver> kungFuDeathGrip(this);
 
-  if (aContent1->Tag() == nsGkAtoms::option) {
-    // update the selected state for this node
-    PRInt32 index = FindContent(aContent1);
-    if (index >= 0)
-      mSelection->ToggleSelect(index);
-  }
-}
-
-void
-nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
-                                    nsIContent*  aContent,
-                                    PRInt32      aNameSpaceID,
-                                    nsIAtom*     aAttribute,
-                                    PRInt32      aModType,
-                                    PRUint32     aStateMask)
-{
   // Make sure this notification concerns us.
   // First check the tag to see if it's one that we care about.
-  nsIAtom *tag = aContent->Tag();
+  nsIAtom* tag = aElement->Tag();
 
-  if (mBoxObject && (aContent == mRoot || aContent == mBody)) {
+  if (mBoxObject && (aElement == mRoot || aElement == mBody)) {
     mBoxObject->ClearStyleAndImageCaches();
     mBoxObject->Invalidate();
   }
 
-  if (aContent->IsNodeOfType(nsINode::eXUL)) {
-    if (tag != nsGkAtoms::treecol &&
-        tag != nsGkAtoms::treeitem &&
-        tag != nsGkAtoms::treeseparator &&
-        tag != nsGkAtoms::treerow &&
-        tag != nsGkAtoms::treecell)
-      return;
-    // We don't consider XUL nodes under non-XUL nodes.
-    if (!aContent->GetParent()->IsNodeOfType(nsINode::eXUL))
-      return;
+  // We don't consider non-XUL nodes.
+  nsIContent* parent = nsnull;
+  if (!aElement->IsXUL() ||
+      ((parent = aElement->GetParent()) && !parent->IsXUL())) {
+    return;
   }
-  else {
+  if (tag != nsGkAtoms::treecol &&
+      tag != nsGkAtoms::treeitem &&
+      tag != nsGkAtoms::treeseparator &&
+      tag != nsGkAtoms::treerow &&
+      tag != nsGkAtoms::treecell) {
     return;
   }
 
   // If we have a legal tag, go up to the tree/select and make sure
   // that it's ours.
 
-  for (nsIContent* element = aContent; element != mBody; element = element->GetParent()) {
+  for (nsIContent* element = aElement; element != mBody; element = element->GetParent()) {
     if (!element)
       return; // this is not for us
     nsIAtom *parentTag = element->Tag();
-    if ((element->IsNodeOfType(nsINode::eXUL) && parentTag == nsGkAtoms::tree) ||
-        (element->IsNodeOfType(nsINode::eHTML) && parentTag == nsGkAtoms::select))
+    if (element->IsXUL() && parentTag == nsGkAtoms::tree)
       return; // this is not for us
   }
 
   // Handle changes of the hidden attribute.
   if (aAttribute == nsGkAtoms::hidden &&
      (tag == nsGkAtoms::treeitem || tag == nsGkAtoms::treeseparator)) {
-    PRBool hidden = aContent->AttrValueIs(kNameSpaceID_None,
+    PRBool hidden = aElement->AttrValueIs(kNameSpaceID_None,
                                           nsGkAtoms::hidden,
                                           nsGkAtoms::_true, eCaseMatters);
  
-    PRInt32 index = FindContent(aContent);
+    PRInt32 index = FindContent(aElement);
     if (hidden && index >= 0) {
       // Hide this row along with its children.
       PRInt32 count = RemoveRow(index);
@@ -882,9 +836,9 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
     }
     else if (!hidden && index < 0) {
       // Show this row along with its children.
-      nsCOMPtr<nsIContent> parent = aContent->GetParent();
+      nsCOMPtr<nsIContent> parent = aElement->GetParent();
       if (parent) {
-        InsertRowFor(parent, aContent);
+        InsertRowFor(parent, aElement);
       }
     }
 
@@ -897,7 +851,7 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
         nsCOMPtr<nsITreeColumns> cols;
         mBoxObject->GetColumns(getter_AddRefs(cols));
         if (cols) {
-          nsCOMPtr<nsIDOMElement> element = do_QueryInterface(aContent);
+          nsCOMPtr<nsIDOMElement> element = do_QueryInterface(aElement);
           nsCOMPtr<nsITreeColumn> col;
           cols->GetColumnFor(element, getter_AddRefs(col));
           mBoxObject->InvalidateColumn(col);
@@ -906,12 +860,12 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
     }
   }
   else if (tag == nsGkAtoms::treeitem) {
-    PRInt32 index = FindContent(aContent);
+    PRInt32 index = FindContent(aElement);
     if (index >= 0) {
       Row* row = mRows[index];
       if (aAttribute == nsGkAtoms::container) {
         PRBool isContainer =
-          aContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::container,
+          aElement->AttrValueIs(kNameSpaceID_None, nsGkAtoms::container,
                                 nsGkAtoms::_true, eCaseMatters);
         row->SetContainer(isContainer);
         if (mBoxObject)
@@ -919,7 +873,7 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
       }
       else if (aAttribute == nsGkAtoms::open) {
         PRBool isOpen =
-          aContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::open,
+          aElement->AttrValueIs(kNameSpaceID_None, nsGkAtoms::open,
                                 nsGkAtoms::_true, eCaseMatters);
         PRBool wasOpen = row->IsOpen();
         if (! isOpen && wasOpen)
@@ -929,7 +883,7 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
       }
       else if (aAttribute == nsGkAtoms::empty) {
         PRBool isEmpty =
-          aContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::empty,
+          aElement->AttrValueIs(kNameSpaceID_None, nsGkAtoms::empty,
                                 nsGkAtoms::_true, eCaseMatters);
         row->SetEmpty(isEmpty);
         if (mBoxObject)
@@ -938,7 +892,7 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
     }
   }
   else if (tag == nsGkAtoms::treeseparator) {
-    PRInt32 index = FindContent(aContent);
+    PRInt32 index = FindContent(aElement);
     if (index >= 0) {
       if (aAttribute == nsGkAtoms::properties && mBoxObject) {
         mBoxObject->InvalidateRow(index);
@@ -947,7 +901,7 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
   }
   else if (tag == nsGkAtoms::treerow) {
     if (aAttribute == nsGkAtoms::properties) {
-      nsCOMPtr<nsIContent> parent = aContent->GetParent();
+      nsCOMPtr<nsIContent> parent = aElement->GetParent();
       if (parent) {
         PRInt32 index = FindContent(parent);
         if (index >= 0 && mBoxObject) {
@@ -963,10 +917,10 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
         aAttribute == nsGkAtoms::src ||
         aAttribute == nsGkAtoms::value ||
         aAttribute == nsGkAtoms::label) {
-      nsIContent* parent = aContent->GetParent();
+      nsIContent* parent = aElement->GetParent();
       if (parent) {
         nsCOMPtr<nsIContent> grandParent = parent->GetParent();
-        if (grandParent && grandParent->IsNodeOfType(nsINode::eXUL)) {
+        if (grandParent && grandParent->IsXUL()) {
           PRInt32 index = FindContent(grandParent);
           if (index >= 0 && mBoxObject) {
             // XXX Should we make an effort to invalidate only cell ?
@@ -981,13 +935,12 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
 void
 nsTreeContentView::ContentAppended(nsIDocument *aDocument,
                                    nsIContent* aContainer,
-                                   PRInt32     aNewIndexInContainer)
+                                   nsIContent* aFirstNewContent,
+                                   PRInt32     /* unused */)
 {
-  PRUint32 childCount = aContainer->GetChildCount();
-  while ((PRUint32)aNewIndexInContainer < childCount) {
-    nsIContent *child = aContainer->GetChildAt(aNewIndexInContainer);
-    ContentInserted(aDocument, aContainer, child, aNewIndexInContainer);
-    aNewIndexInContainer++;
+  for (nsIContent* cur = aFirstNewContent; cur; cur = cur->GetNextSibling()) {
+    // Our contentinserted doesn't use the index
+    ContentInserted(aDocument, aContainer, cur, 0);
   }
 }
 
@@ -995,7 +948,7 @@ void
 nsTreeContentView::ContentInserted(nsIDocument *aDocument,
                                    nsIContent* aContainer,
                                    nsIContent* aChild,
-                                   PRInt32 aIndexInContainer)
+                                   PRInt32 /* unused */)
 {
   NS_ASSERTION(aChild, "null ptr");
 
@@ -1003,23 +956,14 @@ nsTreeContentView::ContentInserted(nsIDocument *aDocument,
   // First check the tag to see if it's one that we care about.
   nsIAtom *childTag = aChild->Tag();
 
-  if (aChild->IsNodeOfType(nsINode::eHTML)) {
-    if (childTag != nsGkAtoms::option &&
-        childTag != nsGkAtoms::optgroup)
-      return;
-  }
-  else if (aChild->IsNodeOfType(nsINode::eXUL)) {
-    if (childTag != nsGkAtoms::treeitem &&
-        childTag != nsGkAtoms::treeseparator &&
-        childTag != nsGkAtoms::treechildren &&
-        childTag != nsGkAtoms::treerow &&
-        childTag != nsGkAtoms::treecell)
-      return;
-    // Don't allow XUL nodes to be inserted under non-XUL nodes.
-    if (!aContainer->IsNodeOfType(nsINode::eXUL))
-      return;
-  }
-  else {
+  // Don't allow non-XUL nodes.
+  if (!aChild->IsXUL() || !aContainer->IsXUL())
+    return;
+  if (childTag != nsGkAtoms::treeitem &&
+      childTag != nsGkAtoms::treeseparator &&
+      childTag != nsGkAtoms::treechildren &&
+      childTag != nsGkAtoms::treerow &&
+      childTag != nsGkAtoms::treecell) {
     return;
   }
 
@@ -1030,11 +974,13 @@ nsTreeContentView::ContentInserted(nsIDocument *aDocument,
     if (!element)
       return; // this is not for us
     nsIAtom *parentTag = element->Tag();
-    if ((element->IsNodeOfType(nsINode::eXUL) && parentTag == nsGkAtoms::tree) ||
-        (element->IsNodeOfType(nsINode::eHTML) && parentTag == nsGkAtoms::select))
+    if (element->IsXUL() && parentTag == nsGkAtoms::tree)
       return; // this is not for us
   }
 
+  // Lots of codepaths under here that do all sorts of stuff, so be safe.
+  nsCOMPtr<nsIMutationObserver> kungFuDeathGrip(this);
+
   if (childTag == nsGkAtoms::treechildren) {
     PRInt32 index = FindContent(aContainer);
     if (index >= 0) {
@@ -1066,27 +1012,14 @@ nsTreeContentView::ContentInserted(nsIDocument *aDocument,
         mBoxObject->InvalidateRow(index);
     }
   }
-  else if (childTag == nsGkAtoms::optgroup) {
-    InsertRowFor(aContainer, aChild);
-  }
-  else if (childTag == nsGkAtoms::option) {
-    PRInt32 parentIndex = FindContent(aContainer);
-
-    if (parentIndex >= 0) {
-      PRInt32 index = 0;
-      GetIndexInSubtree(aContainer, aChild, &index);
-      PRInt32 count = InsertRow(parentIndex, index, aChild);
-      if (mBoxObject)
-        mBoxObject->RowCountChanged(parentIndex + index + 1, count);
-    }
-  }
 }
 
 void
 nsTreeContentView::ContentRemoved(nsIDocument *aDocument,
-                                     nsIContent* aContainer,
-                                     nsIContent* aChild,
-                                     PRInt32 aIndexInContainer)
+                                  nsIContent* aContainer,
+                                  nsIContent* aChild,
+                                  PRInt32 aIndexInContainer,
+                                  nsIContent* aPreviousSibling)
 {
   NS_ASSERTION(aChild, "null ptr");
 
@@ -1094,23 +1027,14 @@ nsTreeContentView::ContentRemoved(nsIDocument *aDocument,
   // First check the tag to see if it's one that we care about.
   nsIAtom *tag = aChild->Tag();
 
-  if (aChild->IsNodeOfType(nsINode::eHTML)) {
-    if (tag != nsGkAtoms::option &&
-        tag != nsGkAtoms::optgroup)
-      return;
-  }
-  else if (aChild->IsNodeOfType(nsINode::eXUL)) {
-    if (tag != nsGkAtoms::treeitem &&
-        tag != nsGkAtoms::treeseparator &&
-        tag != nsGkAtoms::treechildren &&
-        tag != nsGkAtoms::treerow &&
-        tag != nsGkAtoms::treecell)
-      return;
-    // We don't consider XUL nodes under non-XUL nodes.
-    if (!aContainer->IsNodeOfType(nsINode::eXUL))
-      return;
-  }
-  else {
+  // We don't consider non-XUL nodes.
+  if (!aChild->IsXUL() || !aContainer->IsXUL())
+    return;
+  if (tag != nsGkAtoms::treeitem &&
+      tag != nsGkAtoms::treeseparator &&
+      tag != nsGkAtoms::treechildren &&
+      tag != nsGkAtoms::treerow &&
+      tag != nsGkAtoms::treecell) {
     return;
   }
 
@@ -1121,11 +1045,13 @@ nsTreeContentView::ContentRemoved(nsIDocument *aDocument,
     if (!element)
       return; // this is not for us
     nsIAtom *parentTag = element->Tag();
-    if ((element->IsNodeOfType(nsINode::eXUL) && parentTag == nsGkAtoms::tree) ||
-        (element->IsNodeOfType(nsINode::eHTML) && parentTag == nsGkAtoms::select))
+    if (element->IsXUL() && parentTag == nsGkAtoms::tree)
       return; // this is not for us
   }
 
+  // Lots of codepaths under here that do all sorts of stuff, so be safe.
+  nsCOMPtr<nsIMutationObserver> kungFuDeathGrip(this);
+
   if (tag == nsGkAtoms::treechildren) {
     PRInt32 index = FindContent(aContainer);
     if (index >= 0) {
@@ -1140,9 +1066,7 @@ nsTreeContentView::ContentRemoved(nsIDocument *aDocument,
     }
   }
   else if (tag == nsGkAtoms::treeitem ||
-           tag == nsGkAtoms::treeseparator ||
-           tag == nsGkAtoms::option ||
-           tag == nsGkAtoms::optgroup
+           tag == nsGkAtoms::treeseparator
           ) {
     PRInt32 index = FindContent(aChild);
     if (index >= 0) {
@@ -1169,6 +1093,8 @@ nsTreeContentView::ContentRemoved(nsIDocument *aDocument,
 void
 nsTreeContentView::NodeWillBeDestroyed(const nsINode* aNode)
 {
+  // XXXbz do we need this strong ref?  Do we drop refs to self in ClearRows?
+  nsCOMPtr<nsIMutationObserver> kungFuDeathGrip(this);
   ClearRows();
 }
 
@@ -1178,8 +1104,9 @@ void
 nsTreeContentView::Serialize(nsIContent* aContent, PRInt32 aParentIndex,
                              PRInt32* aIndex, nsTArray<Row*>& aRows)
 {
-  // Don't allow XUL nodes under non-XUL nodes.
-  PRBool containerIsXUL = aContent->IsNodeOfType(nsINode::eXUL);
+  // Don't allow non-XUL nodes.
+  if (!aContent->IsXUL())
+    return;
 
   ChildIterator iter, last;
   for (ChildIterator::Init(aContent, &iter, &last); iter != last; ++iter) {
@@ -1187,18 +1114,12 @@ nsTreeContentView::Serialize(nsIContent* aContent, PRInt32 aParentIndex,
     nsIAtom *tag = content->Tag();
     PRInt32 count = aRows.Length();
 
-    if (content->IsNodeOfType(nsINode::eXUL) && containerIsXUL) {
+    if (content->IsXUL()) {
       if (tag == nsGkAtoms::treeitem)
         SerializeItem(content, aParentIndex, aIndex, aRows);
       else if (tag == nsGkAtoms::treeseparator)
         SerializeSeparator(content, aParentIndex, aIndex, aRows);
     }
-    else if (content->IsNodeOfType(nsINode::eHTML)) {
-      if (tag == nsGkAtoms::option)
-        SerializeOption(content, aParentIndex, aIndex, aRows);
-      else if (tag == nsGkAtoms::optgroup)
-        SerializeOptGroup(content, aParentIndex, aIndex, aRows);
-    }
     *aIndex += aRows.Length() - count;
   }
 }
@@ -1222,7 +1143,7 @@ nsTreeContentView::SerializeItem(nsIContent* aContent, PRInt32 aParentIndex,
       row->SetOpen(PR_TRUE);
       nsIContent* child =
         nsTreeUtils::GetImmediateChild(aContent, nsGkAtoms::treechildren);
-      if (child) {
+      if (child && child->IsXUL()) {
         // Now, recursively serialize our child.
         PRInt32 count = aRows.Length();
         PRInt32 index = 0;
@@ -1252,51 +1173,15 @@ nsTreeContentView::SerializeSeparator(nsIContent* aContent,
   aRows.AppendElement(row);
 }
 
-void
-nsTreeContentView::SerializeOption(nsIContent* aContent, PRInt32 aParentIndex,
-                                   PRInt32* aIndex, nsTArray<Row*>& aRows)
-{
-  Row* row = Row::Create(mAllocator, aContent, aParentIndex);
-  aRows.AppendElement(row);
-
-  // This will happen before the TreeSelection is hooked up.  So, cache the selected
-  // state in the row properties and update the selection when it is attached.
-
-  nsCOMPtr<nsIDOMHTMLOptionElement> optEl = do_QueryInterface(aContent);
-  PRBool isSelected;
-  optEl->GetSelected(&isSelected);
-  if (isSelected)
-    mUpdateSelection = PR_TRUE;
-}
-
-void
-nsTreeContentView::SerializeOptGroup(nsIContent* aContent, PRInt32 aParentIndex,
-                                     PRInt32* aIndex, nsTArray<Row*>& aRows)
-{
-  Row* row = Row::Create(mAllocator, aContent, aParentIndex);
-  aRows.AppendElement(row);
-  row->SetContainer(PR_TRUE);
-  row->SetOpen(PR_TRUE);
-
-  nsIContent* child =
-    nsTreeUtils::GetImmediateChild(aContent, nsGkAtoms::option);
-  if (child) {
-    // Now, recursively serialize our child.
-    PRInt32 count = aRows.Length();
-    PRInt32 index = 0;
-    Serialize(aContent, aParentIndex + *aIndex + 1, &index, aRows);
-    row->mSubtreeSize += aRows.Length() - count;
-  }
-  else
-    row->SetEmpty(PR_TRUE);
-}
-
 void
 nsTreeContentView::GetIndexInSubtree(nsIContent* aContainer,
                                      nsIContent* aContent, PRInt32* aIndex)
 {
-  PRBool containerIsXUL = aContainer->IsNodeOfType(nsINode::eXUL);
   PRUint32 childCount = aContainer->GetChildCount();
+  
+  if (!aContainer->IsXUL())
+    return;
+
   for (PRUint32 i = 0; i < childCount; i++) {
     nsIContent *content = aContainer->GetChildAt(i);
 
@@ -1305,7 +1190,7 @@ nsTreeContentView::GetIndexInSubtree(nsIContent* aContainer,
 
     nsIAtom *tag = content->Tag();
 
-    if (content->IsNodeOfType(nsINode::eXUL) && containerIsXUL) {
+    if (content->IsXUL()) {
       if (tag == nsGkAtoms::treeitem) {
         if (! content->AttrValueIs(kNameSpaceID_None, nsGkAtoms::hidden,
                                    nsGkAtoms::_true, eCaseMatters)) {
@@ -1316,7 +1201,7 @@ nsTreeContentView::GetIndexInSubtree(nsIContent* aContainer,
                                    nsGkAtoms::_true, eCaseMatters)) {
             nsIContent* child =
               nsTreeUtils::GetImmediateChild(content, nsGkAtoms::treechildren);
-            if (child)
+            if (child && child->IsXUL())
               GetIndexInSubtree(child, aContent, aIndex);
           }
         }
@@ -1327,14 +1212,6 @@ nsTreeContentView::GetIndexInSubtree(nsIContent* aContainer,
           (*aIndex)++;
       }
     }
-    else if (content->IsNodeOfType(nsINode::eHTML)) {
-      if (tag == nsGkAtoms::optgroup) {
-        (*aIndex)++;
-        GetIndexInSubtree(content, aContent, aIndex);
-      }
-      else if (tag == nsGkAtoms::option)
-        (*aIndex)++;
-    }
   }
 }
 
@@ -1344,14 +1221,9 @@ nsTreeContentView::EnsureSubtree(PRInt32 aIndex)
   Row* row = mRows[aIndex];
 
   nsIContent* child;
-  if (row->mContent->Tag() == nsGkAtoms::optgroup)
-    child = row->mContent;
-  else {
-    child =
-      nsTreeUtils::GetImmediateChild(row->mContent, nsGkAtoms::treechildren);
-    if (! child) {
-      return 0;
-    }
+  child = nsTreeUtils::GetImmediateChild(row->mContent, nsGkAtoms::treechildren);
+  if (!child || !child->IsXUL()) {
+    return 0;
   }
 
   nsAutoTArray<Row*, 8> rows;
@@ -1399,9 +1271,7 @@ nsTreeContentView::InsertRowFor(nsIContent* aParent, nsIContent* aChild)
   nsCOMPtr<nsIContent> grandParent = aParent->GetParent();
   nsIAtom* grandParentTag = grandParent->Tag();
 
-  if ((grandParent->IsNodeOfType(nsINode::eXUL) && grandParentTag == nsGkAtoms::tree) ||
-      (grandParent->IsNodeOfType(nsINode::eHTML) && grandParentTag == nsGkAtoms::select)
-     ) {
+  if (grandParent->IsXUL() && grandParentTag == nsGkAtoms::tree) {
     // Allow insertion to the outermost container.
     insertRow = PR_TRUE;
   }
@@ -1433,18 +1303,12 @@ nsTreeContentView::InsertRow(PRInt32 aParentIndex, PRInt32 aIndex, nsIContent* a
 {
   nsAutoTArray<Row*, 8> rows;
   nsIAtom *tag = aContent->Tag();
-  if (aContent->IsNodeOfType(nsINode::eXUL)) {
+  if (aContent->IsXUL()) {
     if (tag == nsGkAtoms::treeitem)
       SerializeItem(aContent, aParentIndex, &aIndex, rows);
     else if (tag == nsGkAtoms::treeseparator)
       SerializeSeparator(aContent, aParentIndex, &aIndex, rows);
   }
-  else if (aContent->IsNodeOfType(nsINode::eHTML)) {
-    if (tag == nsGkAtoms::option)
-      SerializeOption(aContent, aParentIndex, &aIndex, rows);
-    else if (tag == nsGkAtoms::optgroup)
-      SerializeOptGroup(aContent, aParentIndex, &aIndex, rows);
-  }
 
   mRows.InsertElementsAt(aParentIndex + aIndex + 1, rows);
   PRInt32 count = rows.Length();
