@@ -46,7 +46,6 @@
 #include "nsIServiceManager.h"
 #include "nsReadableUtils.h"
 #include "nsString.h"
-#include "nsAutoLock.h"
 #include "nsAutoPtr.h"
 #include "nsNetCID.h"
 #include "nsNetError.h"
@@ -61,6 +60,8 @@
 
 #include "mozilla/FunctionTimer.h"
 
+using namespace mozilla;
+
 static const char kPrefDnsCacheEntries[]    = "network.dnsCacheEntries";
 static const char kPrefDnsCacheExpiration[] = "network.dnsCacheExpiration";
 static const char kPrefEnableIDN[]          = "network.enableIDN";
@@ -105,13 +106,14 @@ nsDNSRecord::GetCanonicalName(nsACString &result)
     // if the record is for an IP address literal, then the canonical
     // host name is the IP address literal.
     const char *cname;
-    PR_Lock(mHostRecord->addr_info_lock);
-    if (mHostRecord->addr_info)
-        cname = PR_GetCanonNameFromAddrInfo(mHostRecord->addr_info);
-    else
-        cname = mHostRecord->host;
-    result.Assign(cname);
-    PR_Unlock(mHostRecord->addr_info_lock);
+    {
+        MutexAutoLock lock(*mHostRecord->addr_info_lock);
+        if (mHostRecord->addr_info)
+            cname = PR_GetCanonNameFromAddrInfo(mHostRecord->addr_info);
+        else
+            cname = mHostRecord->host;
+        result.Assign(cname);
+    }
     return NS_OK;
 }
 
@@ -124,7 +126,7 @@ nsDNSRecord::GetNextAddr(PRUint16 port, PRNetAddr *addr)
     if (mDone)
         return NS_ERROR_NOT_AVAILABLE;
 
-    PR_Lock(mHostRecord->addr_info_lock);
+    mHostRecord->addr_info_lock->Lock();
     if (mHostRecord->addr_info) {
         if (!mIter)
             mIterGenCnt = mHostRecord->addr_info_gencnt;
@@ -135,14 +137,14 @@ nsDNSRecord::GetNextAddr(PRUint16 port, PRNetAddr *addr)
             mIterGenCnt = mHostRecord->addr_info_gencnt;
         }
         mIter = PR_EnumerateAddrInfo(mIter, mHostRecord->addr_info, port, addr);
-        PR_Unlock(mHostRecord->addr_info_lock);
+        mHostRecord->addr_info_lock->Unlock();
         if (!mIter) {
             mDone = PR_TRUE;
             return NS_ERROR_NOT_AVAILABLE;
         }
     }
     else {
-        PR_Unlock(mHostRecord->addr_info_lock);
+        mHostRecord->addr_info_lock->Unlock();
         if (!mHostRecord->addr) {
             // Both mHostRecord->addr_info and mHostRecord->addr are null.
             // This can happen if mHostRecord->addr_info expired and the
@@ -306,14 +308,13 @@ nsDNSSyncRequest::OnLookupComplete(nsHostResolver *resolver,
 //-----------------------------------------------------------------------------
 
 nsDNSService::nsDNSService()
-    : mLock(nsnull)
+    : mLock("nsDNSServer.mLock")
+    , mFirstTime(PR_TRUE)
 {
 }
 
 nsDNSService::~nsDNSService()
 {
-    if (mLock)
-        PR_DestroyLock(mLock);
 }
 
 NS_IMPL_THREADSAFE_ISUPPORTS3(nsDNSService, nsIDNSService, nsPIDNSService,
@@ -326,8 +327,6 @@ nsDNSService::Init()
 
     NS_ENSURE_TRUE(!mResolver, NS_ERROR_ALREADY_INITIALIZED);
 
-    PRBool firstTime = (mLock == nsnull);
-
     // prefs
     PRUint32 maxCacheEntries  = 400;
     PRUint32 maxCacheLifetime = 3; // minutes
@@ -357,10 +356,8 @@ nsDNSService::Init()
         prefs->GetIntPref("network.proxy.type", &proxyType);
     }
 
-    if (firstTime) {
-        mLock = PR_NewLock();
-        if (!mLock)
-            return NS_ERROR_OUT_OF_MEMORY;
+    if (mFirstTime) {
+        mFirstTime = PR_FALSE;
 
         // register as prefs observer
         if (prefs) {
@@ -396,7 +393,7 @@ nsDNSService::Init()
                                          getter_AddRefs(res));
     if (NS_SUCCEEDED(rv)) {
         // now, set all of our member variables while holding the lock
-        nsAutoLock lock(mLock);
+        MutexAutoLock lock(mLock);
         mResolver = res;
         mIDN = idn;
         mIPv4OnlyDomains = ipv4OnlyDomains; // exchanges buffer ownership
@@ -413,7 +410,7 @@ nsDNSService::Shutdown()
 {
     nsRefPtr<nsHostResolver> res;
     {
-        nsAutoLock lock(mLock);
+        MutexAutoLock lock(mLock);
         res = mResolver;
         mResolver = nsnull;
     }
@@ -434,7 +431,7 @@ nsDNSService::AsyncResolve(const nsACString  &hostname,
     nsRefPtr<nsHostResolver> res;
     nsCOMPtr<nsIIDNService> idn;
     {
-        nsAutoLock lock(mLock);
+        MutexAutoLock lock(mLock);
 
         if (mDisablePrefetch && (flags & RESOLVE_SPECULATE))
             return NS_ERROR_DNS_LOOKUP_QUEUE_FULL;
@@ -493,7 +490,7 @@ nsDNSService::Resolve(const nsACString &hostname,
     nsRefPtr<nsHostResolver> res;
     nsCOMPtr<nsIIDNService> idn;
     {
-        nsAutoLock lock(mLock);
+        MutexAutoLock lock(mLock);
         res = mResolver;
         idn = mIDN;
     }
@@ -588,7 +585,7 @@ nsDNSService::GetAFForLookup(const nsACString &host)
     if (mDisableIPv6)
         return PR_AF_INET;
 
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
 
     PRUint16 af = PR_AF_UNSPEC;
 
