@@ -50,17 +50,29 @@
 #include "nsCOMPtr.h"
 #include "nsIAtom.h"
 #include "nsIDOMKeyEvent.h"
+#include "nsIDOMNSMouseEvent.h"
 #include "nsIDOMDataTransfer.h"
+#include "nsPIDOMEventTarget.h"
 #include "nsWeakPtr.h"
 #include "nsIWidget.h"
 #include "nsTArray.h"
 #include "nsTraceRefcnt.h"
 #include "nsITransferable.h"
+#include "nsIVariant.h"
+#include "nsStyleConsts.h"
 
-class nsIRenderingContext;
-class nsIRegion;
+namespace mozilla {
+namespace dom {
+  class PBrowserParent;
+  class PBrowserChild;
+}
+}
+
+#ifdef ACCESSIBILITY
+class nsAccessible;
+#endif
+class nsRenderingContext;
 class nsIMenuItem;
-class nsIAccessible;
 class nsIContent;
 class nsIURI;
 class nsHashKey;
@@ -78,7 +90,6 @@ class nsHashKey;
 #define NS_INPUT_EVENT                     8
 #define NS_KEY_EVENT                       9
 #define NS_MOUSE_EVENT                    10
-#define NS_MENU_EVENT                     11
 #define NS_SCRIPT_ERROR_EVENT             12
 #define NS_TEXT_EVENT                     13
 #define NS_COMPOSITION_EVENT              14
@@ -87,27 +98,35 @@ class nsHashKey;
 #define NS_MUTATION_EVENT                 19 // |nsMutationEvent| in content
 #define NS_ACCESSIBLE_EVENT               20
 #define NS_FORM_EVENT                     21
+#define NS_FOCUS_EVENT                    22
 #define NS_POPUP_EVENT                    23
 #define NS_COMMAND_EVENT                  24
 #define NS_SCROLLAREA_EVENT               25
+#define NS_TRANSITION_EVENT               26
+#ifdef MOZ_CSS_ANIMATIONS
+#define NS_ANIMATION_EVENT                27
+#endif
 
-
-#define NS_UI_EVENT                       27
+#define NS_UI_EVENT                       28
 #ifdef MOZ_SVG
 #define NS_SVG_EVENT                      30
 #define NS_SVGZOOM_EVENT                  31
 #endif // MOZ_SVG
+#ifdef MOZ_SMIL
+#define NS_SMIL_TIME_EVENT                32
+#endif // MOZ_SMIL
 
 #define NS_QUERY_CONTENT_EVENT            33
-#ifdef MOZ_MEDIA
-#define NS_MEDIA_EVENT                    34
-#endif // MOZ_MEDIA
+
 #define NS_DRAG_EVENT                     35
 #define NS_NOTIFYPAINT_EVENT              36
 #define NS_SIMPLE_GESTURE_EVENT           37
 #define NS_SELECTION_EVENT                38
 #define NS_CONTENT_COMMAND_EVENT          39
 #define NS_GESTURENOTIFY_EVENT            40
+#define NS_UISTATECHANGE_EVENT            41
+#define NS_MOZTOUCH_EVENT                 42
+#define NS_PLUGIN_EVENT                   43
 
 // These flags are sort of a mess. They're sort of shared between event
 // listener flags and event flags, but only some of them. You've been
@@ -127,10 +146,10 @@ class nsHashKey;
 #define NS_EVENT_DISPATCHED               0x0400
 #define NS_EVENT_FLAG_DISPATCHING         0x0800
 // When an event is synthesized for testing, this flag will be set.
-// Note that this is currently used only with mouse events.  Because this flag
-// is not needed on other events now.  Therfore, if you need this flag on other
-// events, you can do it.
-#define NS_EVENT_FLAG_SYNTETIC_TEST_EVENT 0x1000
+// Note that this is currently used only with mouse events, because this
+// flag is not needed on other events now.  It could be added to other
+// events.
+#define NS_EVENT_FLAG_SYNTHETIC_TEST_EVENT 0x1000
 
 // Use this flag if the event should be dispatched only to chrome.
 #define NS_EVENT_FLAG_ONLY_CHROME_DISPATCH 0x2000
@@ -140,6 +159,10 @@ class nsHashKey;
 
 #define NS_PRIV_EVENT_UNTRUSTED_PERMITTED 0x8000
 
+#define NS_EVENT_FLAG_EXCEPTION_THROWN    0x10000
+
+#define NS_EVENT_FLAG_PREVENT_ANCHOR_ACTIONS 0x20000
+
 #define NS_EVENT_CAPTURE_MASK             (~(NS_EVENT_FLAG_BUBBLE | NS_EVENT_FLAG_NO_CONTENT_DISPATCH))
 #define NS_EVENT_BUBBLE_MASK              (~(NS_EVENT_FLAG_CAPTURE | NS_EVENT_FLAG_NO_CONTENT_DISPATCH))
 
@@ -170,6 +193,9 @@ class nsHashKey;
 #define NS_DEACTIVATE                   (NS_WINDOW_START + 8)
 // top-level window z-level change request
 #define NS_SETZLEVEL                    (NS_WINDOW_START + 9)
+// Widget was repainted (dispatched when it's safe to move widgets, but
+// only on some platforms (including GTK2 and Windows))
+#define NS_DID_PAINT                   (NS_WINDOW_START + 28)
 // Widget will need to be painted
 #define NS_WILL_PAINT                   (NS_WINDOW_START + 29)
 // Widget needs to be repainted
@@ -189,15 +215,6 @@ class nsHashKey;
 
 #define NS_OS_TOOLBAR                   (NS_WINDOW_START + 36)
 
-// Menu item selected
-#define NS_MENU_SELECTED                (NS_WINDOW_START + 38)
-
-// Form control changed: currently == combo box selection changed
-// but could be expanded to mean textbox, checkbox changed, etc.
-// This is a GUI specific event that does not necessarily correspond
-// directly to a mouse click or a key press.
-#define NS_CONTROL_CHANGE                (NS_WINDOW_START + 39)
-
 // Indicates the display has changed depth
 #define NS_DISPLAYCHANGED                (NS_WINDOW_START + 40)
 
@@ -209,13 +226,28 @@ class nsHashKey;
 // ensure that it is drawn using the current system colors.
 #define NS_SYSCOLORCHANGED              (NS_WINDOW_START + 42)
 
+// Indicates that the ui state such as whether to show focus or
+// keyboard accelerator indicators has changed.
+#define NS_UISTATECHANGED               (NS_WINDOW_START + 43)
+
+// Done sizing or moving a window, so ensure that the mousedown state was cleared.
+#define NS_DONESIZEMOVE                 (NS_WINDOW_START + 44)
+
 #define NS_RESIZE_EVENT                 (NS_WINDOW_START + 60)
 #define NS_SCROLL_EVENT                 (NS_WINDOW_START + 61)
 
+// A plugin was clicked or otherwise focused. NS_PLUGIN_ACTIVATE should be
+// used when the window is not active. NS_PLUGIN_FOCUS should be used when
+// the window is active. In the latter case, the dispatcher of the event
+// is expected to ensure that the plugin's widget is focused beforehand.
 #define NS_PLUGIN_ACTIVATE               (NS_WINDOW_START + 62)
+#define NS_PLUGIN_FOCUS                  (NS_WINDOW_START + 63)
 
-#define NS_OFFLINE                       (NS_WINDOW_START + 63)
-#define NS_ONLINE                        (NS_WINDOW_START + 64)
+#define NS_OFFLINE                       (NS_WINDOW_START + 64)
+#define NS_ONLINE                        (NS_WINDOW_START + 65)
+
+// Indicates a resize will occur
+#define NS_BEFORERESIZE_EVENT            (NS_WINDOW_START + 66)
 
 #define NS_MOUSE_MESSAGE_START          300
 #define NS_MOUSE_MOVE                   (NS_MOUSE_MESSAGE_START)
@@ -228,6 +260,7 @@ class nsHashKey;
 #define NS_MOUSE_ACTIVATE               (NS_MOUSE_MESSAGE_START + 30)
 #define NS_MOUSE_ENTER_SYNTH            (NS_MOUSE_MESSAGE_START + 31)
 #define NS_MOUSE_EXIT_SYNTH             (NS_MOUSE_MESSAGE_START + 32)
+#define NS_MOUSE_MOZHITTEST             (NS_MOUSE_MESSAGE_START + 33)
 
 #define NS_CONTEXTMENU_MESSAGE_START    500
 #define NS_CONTEXTMENU                  (NS_CONTEXTMENU_MESSAGE_START)
@@ -245,8 +278,10 @@ class nsHashKey;
 #define NS_HASHCHANGE                   (NS_STREAM_EVENT_START + 2)
 #define NS_IMAGE_ABORT                  (NS_STREAM_EVENT_START + 3)
 #define NS_LOAD_ERROR                   (NS_STREAM_EVENT_START + 4)
+#define NS_POPSTATE                     (NS_STREAM_EVENT_START + 5)
 #define NS_BEFORE_PAGE_UNLOAD           (NS_STREAM_EVENT_START + 6)
 #define NS_PAGE_RESTORE                 (NS_STREAM_EVENT_START + 7)
+#define NS_READYSTATECHANGE             (NS_STREAM_EVENT_START + 8)
  
 #define NS_FORM_EVENT_START             1200
 #define NS_FORM_SUBMIT                  (NS_FORM_EVENT_START)
@@ -254,6 +289,7 @@ class nsHashKey;
 #define NS_FORM_CHANGE                  (NS_FORM_EVENT_START + 2)
 #define NS_FORM_SELECTED                (NS_FORM_EVENT_START + 3)
 #define NS_FORM_INPUT                   (NS_FORM_EVENT_START + 4)
+#define NS_FORM_INVALID                 (NS_FORM_EVENT_START + 5)
 
 //Need separate focus/blur notifications for non-native widgets
 #define NS_FOCUS_EVENT_START            1300
@@ -307,7 +343,6 @@ class nsHashKey;
 #define NS_COMPOSITION_EVENT_START    2200
 #define NS_COMPOSITION_START          (NS_COMPOSITION_EVENT_START)
 #define NS_COMPOSITION_END            (NS_COMPOSITION_EVENT_START + 1)
-#define NS_COMPOSITION_QUERY          (NS_COMPOSITION_EVENT_START + 2)
 
 // text events
 #define NS_TEXT_START                 2400
@@ -377,6 +412,15 @@ class nsHashKey;
 // Query for character at a point.  This returns the character offset and its
 // rect.  The point is specified by nsEvent::refPoint.
 #define NS_QUERY_CHARACTER_AT_POINT     (NS_QUERY_CONTENT_EVENT_START + 8)
+// Query if the DOM element under nsEvent::refPoint belongs to our widget
+// or not.
+#define NS_QUERY_DOM_WIDGET_HITTEST     (NS_QUERY_CONTENT_EVENT_START + 9)
+// Query for some information about mouse wheel event's target
+// XXX This is used only for supporting high resolution mouse scroll on Windows
+//     and it's going to be reimplemented with another approach.  At that time,
+//     this even is going to be removed. Therefore,  DON'T use this event for
+//     other purpose.
+#define NS_QUERY_SCROLL_TARGET_INFO     (NS_QUERY_CONTENT_EVENT_START + 99)
 
 // Video events
 #ifdef MOZ_MEDIA
@@ -401,13 +445,13 @@ class nsHashKey;
 #define NS_RATECHANGE          (NS_MEDIA_EVENT_START+17)
 #define NS_DURATIONCHANGE      (NS_MEDIA_EVENT_START+18)
 #define NS_VOLUMECHANGE        (NS_MEDIA_EVENT_START+19)
-#define NS_MEDIA_ABORT         (NS_MEDIA_EVENT_START+20)
-#define NS_MEDIA_ERROR         (NS_MEDIA_EVENT_START+21)
+#define NS_MOZAUDIOAVAILABLE   (NS_MEDIA_EVENT_START+20)
 #endif // MOZ_MEDIA
 
 // paint notification events
 #define NS_NOTIFYPAINT_START    3400
 #define NS_AFTERPAINT           (NS_NOTIFYPAINT_START)
+#define NS_BEFOREPAINT          (NS_NOTIFYPAINT_START+1)
 
 // Simple gesture events
 #define NS_SIMPLE_GESTURE_EVENT_START    3500
@@ -421,10 +465,10 @@ class nsHashKey;
 #define NS_SIMPLE_GESTURE_TAP            (NS_SIMPLE_GESTURE_EVENT_START+7)
 #define NS_SIMPLE_GESTURE_PRESSTAP       (NS_SIMPLE_GESTURE_EVENT_START+8)
 
-// Plug-in event. This is used when a plug-in has focus and when the native
-// event needs to be passed to the focused plug-in directly.
-#define NS_PLUGIN_EVENT_START   3600
-#define NS_PLUGIN_EVENT         (NS_PLUGIN_EVENT_START)
+// These are used to send native events to plugins.
+#define NS_PLUGIN_EVENT_START            3600
+#define NS_PLUGIN_INPUT_EVENT            (NS_PLUGIN_EVENT_START)
+#define NS_PLUGIN_FOCUS_EVENT            (NS_PLUGIN_EVENT_START+1)
 
 // Events to manipulate selection (nsSelectionEvent)
 #define NS_SELECTION_EVENT_START        3700
@@ -439,6 +483,14 @@ class nsHashKey;
 #define NS_CONTENT_COMMAND_DELETE       (NS_CONTENT_COMMAND_EVENT_START+3)
 #define NS_CONTENT_COMMAND_UNDO         (NS_CONTENT_COMMAND_EVENT_START+4)
 #define NS_CONTENT_COMMAND_REDO         (NS_CONTENT_COMMAND_EVENT_START+5)
+#define NS_CONTENT_COMMAND_PASTE_TRANSFERABLE (NS_CONTENT_COMMAND_EVENT_START+6)
+// NS_CONTENT_COMMAND_SCROLL scrolls the nearest scrollable element to the
+// currently focused content or latest DOM selection. This would normally be
+// the same element scrolled by keyboard scroll commands, except that this event
+// will scroll an element scrollable in either direction.  I.e., if the nearest
+// scrollable ancestor element can only be scrolled vertically, and horizontal
+// scrolling is requested using this event, no scrolling will occur.
+#define NS_CONTENT_COMMAND_SCROLL       (NS_CONTENT_COMMAND_EVENT_START+7)
 
 // Event to gesture notification
 #define NS_GESTURENOTIFY_EVENT_START 3900
@@ -448,6 +500,49 @@ class nsHashKey;
 #define NS_SCROLLAREA_EVENT_START    4100
 #define NS_SCROLLEDAREACHANGED       (NS_SCROLLAREA_EVENT_START)
 
+#define NS_TRANSITION_EVENT_START    4200
+#define NS_TRANSITION_END            (NS_TRANSITION_EVENT_START)
+
+#ifdef MOZ_CSS_ANIMATIONS
+#define NS_ANIMATION_EVENT_START     4250
+#define NS_ANIMATION_START           (NS_ANIMATION_EVENT_START)
+#define NS_ANIMATION_END             (NS_ANIMATION_EVENT_START + 1)
+#define NS_ANIMATION_ITERATION       (NS_ANIMATION_EVENT_START + 2)
+#endif
+
+#ifdef MOZ_SMIL
+#define NS_SMIL_TIME_EVENT_START     4300
+#define NS_SMIL_BEGIN                (NS_SMIL_TIME_EVENT_START)
+#define NS_SMIL_END                  (NS_SMIL_TIME_EVENT_START + 1)
+#define NS_SMIL_REPEAT               (NS_SMIL_TIME_EVENT_START + 2)
+#endif // MOZ_SMIL
+
+#define NS_MOZTOUCH_EVENT_START      4400
+#define NS_MOZTOUCH_DOWN             (NS_MOZTOUCH_EVENT_START)
+#define NS_MOZTOUCH_MOVE             (NS_MOZTOUCH_EVENT_START+1)
+#define NS_MOZTOUCH_UP               (NS_MOZTOUCH_EVENT_START+2)
+
+// script notification events
+#define NS_NOTIFYSCRIPT_START        4500
+#define NS_BEFORE_SCRIPT_EXECUTE     (NS_NOTIFYSCRIPT_START)
+#define NS_AFTER_SCRIPT_EXECUTE      (NS_NOTIFYSCRIPT_START+1)
+
+#define NS_PRINT_EVENT_START         4600
+#define NS_BEFOREPRINT               (NS_PRINT_EVENT_START)
+#define NS_AFTERPRINT                (NS_PRINT_EVENT_START + 1)
+
+#define NS_MESSAGE_EVENT_START       4700
+#define NS_MESSAGE                   (NS_MESSAGE_EVENT_START)
+
+// Open and close events
+#define NS_OPENCLOSE_EVENT_START     4800
+#define NS_OPEN                      (NS_OPENCLOSE_EVENT_START)
+#define NS_CLOSE                     (NS_OPENCLOSE_EVENT_START+1)
+
+// Device motion and orientation
+#define NS_DEVICE_ORIENTATION_START  4900
+#define NS_DEVICE_ORIENTATION        (NS_DEVICE_ORIENTATION_START)
+#define NS_DEVICE_MOTION             (NS_DEVICE_ORIENTATION_START+1)
 
 /**
  * Return status for event processors, nsEventStatus, is defined in
@@ -481,6 +576,10 @@ protected:
     MOZ_COUNT_CTOR(nsEvent);
   }
 
+  nsEvent()
+  {
+  }
+
 public:
   nsEvent(PRBool isTrusted, PRUint32 msg)
     : eventStructType(NS_EVENT),
@@ -502,22 +601,21 @@ public:
   PRUint8     eventStructType;
   // See GUI MESSAGES,
   PRUint32    message;
-  // In widget relative coordinates, not modified by layout code.
+  // Relative to the widget of the event, or if there is no widget then it is
+  // in screen coordinates. Not modified by layout code.
   nsIntPoint  refPoint;
   // Elapsed time, in milliseconds, from a platform-specific zero time
   // to the time the message was created
-  PRUint32    time;
+  PRUint64    time;
   // Flags to hold event flow stage and capture/bubble cancellation
   // status. This is used also to indicate whether the event is trusted.
   PRUint32    flags;
   // Additional type info for user defined events
   nsCOMPtr<nsIAtom>     userType;
   // Event targets, needed by DOM Events
-  // Using nsISupports, not nsIDOMEventTarget because in some cases
-  // nsIDOMEventTarget is implemented as a tearoff.
-  nsCOMPtr<nsISupports> target;
-  nsCOMPtr<nsISupports> currentTarget;
-  nsCOMPtr<nsISupports> originalTarget;
+  nsCOMPtr<nsPIDOMEventTarget> target;
+  nsCOMPtr<nsPIDOMEventTarget> currentTarget;
+  nsCOMPtr<nsPIDOMEventTarget> originalTarget;
 };
 
 /**
@@ -529,22 +627,27 @@ class nsGUIEvent : public nsEvent
 protected:
   nsGUIEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w, PRUint8 structType)
     : nsEvent(isTrusted, msg, structType),
-      widget(w), nativeMsg(nsnull)
+      widget(w), pluginEvent(nsnull)
+  {
+  }
+
+  nsGUIEvent()
+    : pluginEvent(nsnull)
   {
   }
 
 public:
   nsGUIEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
     : nsEvent(isTrusted, msg, NS_GUI_EVENT),
-      widget(w), nativeMsg(nsnull)
+      widget(w), pluginEvent(nsnull)
   {
   }
 
   /// Originator of the event
   nsCOMPtr<nsIWidget> widget;
 
-  /// Internal platform specific message.
-  void* nativeMsg;
+  /// Event for NPAPI plugin
+  void* pluginEvent;
 };
 
 /**
@@ -632,16 +735,13 @@ class nsPaintEvent : public nsGUIEvent
 public:
   nsPaintEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
     : nsGUIEvent(isTrusted, msg, w, NS_PAINT_EVENT),
-      renderingContext(nsnull), region(nsnull), rect(nsnull)
+      willSendDidPaint(PR_FALSE)
   {
   }
 
-  /// Context to paint in.
-  nsIRenderingContext *renderingContext;
-  /// area to paint  (should be used instead of rect)
-  nsIRegion           *region;
-  /// x,y, width, height in pixels of area to paint
-  nsIntRect           *rect;
+  // area that needs repainting
+  nsIntRegion region;
+  PRPackedBool willSendDidPaint;
 };
 
 /**
@@ -700,6 +800,10 @@ protected:
   {
   }
 
+  nsInputEvent()
+  {
+  }
+
 public:
   nsInputEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
     : nsGUIEvent(isTrusted, msg, w, NS_INPUT_EVENT),
@@ -725,7 +829,8 @@ class nsMouseEvent_base : public nsInputEvent
 {
 public:
   nsMouseEvent_base(PRBool isTrusted, PRUint32 msg, nsIWidget *w, PRUint8 type)
-  : nsInputEvent(isTrusted, msg, w, type), button(0), pressure(0) {}
+  : nsInputEvent(isTrusted, msg, w, type), button(0), pressure(0),
+    inputSource(nsIDOMNSMouseEvent::MOZ_SOURCE_MOUSE) {}
 
   /// The possible related target
   nsCOMPtr<nsISupports> relatedTarget;
@@ -735,6 +840,9 @@ public:
   // Finger or touch pressure of event
   // ranges between 0.0 and 1.0
   float                 pressure;
+
+  // Possible values at nsIDOMNSMouseEvent
+  PRUint16              inputSource;
 };
 
 class nsMouseEvent : public nsMouseEvent_base
@@ -817,6 +925,7 @@ public:
   PRPackedBool userCancelled;
 };
 
+#ifdef ACCESSIBILITY
 /**
  * Accessible event
  */
@@ -826,12 +935,13 @@ class nsAccessibleEvent : public nsInputEvent
 public:
   nsAccessibleEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
     : nsInputEvent(isTrusted, msg, w, NS_ACCESSIBLE_EVENT),
-      accessible(nsnull)
+      mAccessible(nsnull)
   {
   }
 
-  nsIAccessible*  accessible;     
+  nsAccessible *mAccessible;
 };
+#endif
 
 /**
  * Keyboard event
@@ -874,12 +984,12 @@ public:
 struct nsTextRangeStyle
 {
   enum {
-    LINESTYLE_NONE   = 0,
-    LINESTYLE_SOLID  = 1,
-    LINESTYLE_DOTTED = 2,
-    LINESTYLE_DASHED = 3,
-    LINESTYLE_DOUBLE = 4,
-    LINESTYLE_WAVY   = 5
+    LINESTYLE_NONE   = NS_STYLE_TEXT_DECORATION_STYLE_NONE,
+    LINESTYLE_SOLID  = NS_STYLE_TEXT_DECORATION_STYLE_SOLID,
+    LINESTYLE_DOTTED = NS_STYLE_TEXT_DECORATION_STYLE_DOTTED,
+    LINESTYLE_DASHED = NS_STYLE_TEXT_DECORATION_STYLE_DASHED,
+    LINESTYLE_DOUBLE = NS_STYLE_TEXT_DECORATION_STYLE_DOUBLE,
+    LINESTYLE_WAVY   = NS_STYLE_TEXT_DECORATION_STYLE_WAVY
   };
 
   enum {
@@ -988,22 +1098,19 @@ struct nsTextRange
 
 typedef nsTextRange* nsTextRangeArray;
 
-struct nsTextEventReply
+class nsTextEvent : public nsInputEvent
 {
-  nsTextEventReply()
-    : mCursorIsCollapsed(PR_FALSE), mReferenceWidget(nsnull)
+private:
+  friend class mozilla::dom::PBrowserParent;
+  friend class mozilla::dom::PBrowserChild;
+
+  nsTextEvent()
   {
   }
 
-  nsIntRect mCursorPosition;
-  PRBool mCursorIsCollapsed;
-  nsIWidget* mReferenceWidget;
-};
+public:
+  PRUint32 seqno;
 
-typedef struct nsTextEventReply nsTextEventReply;
-
-class nsTextEvent : public nsInputEvent
-{
 public:
   nsTextEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
     : nsInputEvent(isTrusted, msg, w, NS_TEXT_EVENT),
@@ -1012,7 +1119,6 @@ public:
   }
 
   nsString          theText;
-  nsTextEventReply  theReply;
   PRUint32          rangeCount;
   // Note that the range array may not specify a caret position; in that
   // case there will be no range of type NS_TEXTRANGE_CARETPOSITION in the
@@ -1023,13 +1129,22 @@ public:
 
 class nsCompositionEvent : public nsInputEvent
 {
+private:
+  friend class mozilla::dom::PBrowserParent;
+  friend class mozilla::dom::PBrowserChild;
+
+  nsCompositionEvent()
+  {
+  }
+
+public:
+  PRUint32 seqno;
+
 public:
   nsCompositionEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
     : nsInputEvent(isTrusted, msg, w, NS_COMPOSITION_EVENT)
   {
   }
-
-  nsTextEventReply theReply;
 };
 
 /* Mouse Scroll Events: Line Scrolling, Pixel Scrolling and Common Event Flows
@@ -1090,8 +1205,13 @@ public:
                             // will compute the appropriate height/width based on
                             // view lineHeight and generate line scroll events
                             // as needed.
-    kNoDefer =      1 << 5  // For scrollable views, indicates scroll should not
+    kNoDefer =      1 << 5, // For scrollable views, indicates scroll should not
                             // occur asynchronously.
+    kIsMomentum =   1 << 6, // Marks scroll events that aren't controlled by the
+                            // user but fire automatically as the result of a
+                            // "momentum" scroll.
+    kAllowSmoothScroll = 1 << 7 // Allow smooth scroll for the pixel scroll
+                                // event.
   };
 
   nsMouseScrollEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
@@ -1139,10 +1259,20 @@ public:
 
 class nsQueryContentEvent : public nsGUIEvent
 {
+private:
+  friend class mozilla::dom::PBrowserParent;
+  friend class mozilla::dom::PBrowserChild;
+
+  nsQueryContentEvent()
+  {
+    mReply.mContentsRoot = nsnull;
+    mReply.mFocusedWidget = nsnull;
+  }
+
 public:
   nsQueryContentEvent(PRBool aIsTrusted, PRUint32 aMsg, nsIWidget *aWidget) :
     nsGUIEvent(aIsTrusted, aMsg, aWidget, NS_QUERY_CONTENT_EVENT),
-    mSucceeded(PR_FALSE)
+    mSucceeded(PR_FALSE), mWasAsync(PR_FALSE)
   {
   }
 
@@ -1169,10 +1299,41 @@ public:
     mInput.mLength = aLength;
   }
 
+  void InitForQueryDOMWidgetHittest(nsIntPoint& aPoint)
+  {
+    NS_ASSERTION(message == NS_QUERY_DOM_WIDGET_HITTEST,
+                 "wrong initializer is called");
+    refPoint = aPoint;
+  }
+
+  void InitForQueryScrollTargetInfo(nsMouseScrollEvent* aEvent)
+  {
+    NS_ASSERTION(message == NS_QUERY_SCROLL_TARGET_INFO,
+                 "wrong initializer is called");
+    mInput.mMouseScrollEvent = aEvent;
+  }
+
+  PRUint32 GetSelectionStart(void) const
+  {
+    NS_ASSERTION(message == NS_QUERY_SELECTED_TEXT,
+                 "not querying selection");
+    return mReply.mOffset + (mReply.mReversed ? mReply.mString.Length() : 0);
+  }
+
+  PRUint32 GetSelectionEnd(void) const
+  {
+    NS_ASSERTION(message == NS_QUERY_SELECTED_TEXT,
+                 "not querying selection");
+    return mReply.mOffset + (mReply.mReversed ? 0 : mReply.mString.Length());
+  }
+
   PRBool mSucceeded;
+  PRPackedBool mWasAsync;
   struct {
     PRUint32 mOffset;
     PRUint32 mLength;
+    // used by NS_QUERY_SCROLL_TARGET_INFO
+    nsMouseScrollEvent* mMouseScrollEvent;
   } mInput;
   struct {
     void* mContentsRoot;
@@ -1183,8 +1344,20 @@ public:
     nsIWidget* mFocusedWidget;
     PRPackedBool mReversed; // true if selection is reversed (end < start)
     PRPackedBool mHasSelection; // true if the selection exists
+    PRPackedBool mWidgetIsHit; // true if DOM element under mouse belongs to widget
     // used by NS_QUERY_SELECTION_AS_TRANSFERABLE
     nsCOMPtr<nsITransferable> mTransferable;
+    // used by NS_QUERY_SCROLL_TARGET_INFO
+    PRInt32 mLineHeight;
+    PRInt32 mPageWidth;
+    PRInt32 mPageHeight;
+    // used by NS_QUERY_SCROLL_TARGET_INFO
+    // the mouse wheel scrolling amount may be overridden by prefs or
+    // overriding system scrolling speed mechanism.
+    // If mMouseScrollEvent is a line scroll event, the unit of this value is
+    // line.  If mMouseScrollEvent is a page scroll event, the unit of this
+    // value is page.
+    PRInt32 mComputedScrollAmount;
   } mReply;
 
   enum {
@@ -1192,18 +1365,44 @@ public:
   };
 };
 
+class nsFocusEvent : public nsEvent
+{
+public:
+  nsFocusEvent(PRBool isTrusted, PRUint32 msg)
+    : nsEvent(isTrusted, msg, NS_FOCUS_EVENT),
+      fromRaise(PR_FALSE),
+      isRefocus(PR_FALSE)
+  {
+  }
+
+  PRPackedBool fromRaise;
+  PRPackedBool isRefocus;
+};
+
 class nsSelectionEvent : public nsGUIEvent
 {
+private:
+  friend class mozilla::dom::PBrowserParent;
+  friend class mozilla::dom::PBrowserChild;
+
+  nsSelectionEvent()
+  {
+  }
+
+public:
+  PRUint32 seqno;
+
 public:
   nsSelectionEvent(PRBool aIsTrusted, PRUint32 aMsg, nsIWidget *aWidget) :
     nsGUIEvent(aIsTrusted, aMsg, aWidget, NS_SELECTION_EVENT),
-    mSucceeded(PR_FALSE)
+    mExpandToClusterBoundary(PR_TRUE), mSucceeded(PR_FALSE)
   {
   }
 
   PRUint32 mOffset; // start offset of selection
   PRUint32 mLength; // length of selection
   PRPackedBool mReversed; // selection "anchor" should be in front
+  PRPackedBool mExpandToClusterBoundary; // cluster-based or character-based
   PRPackedBool mSucceeded;
 };
 
@@ -1218,30 +1417,45 @@ public:
   {
   }
 
+  // NS_CONTENT_COMMAND_PASTE_TRANSFERABLE
+  nsCOMPtr<nsITransferable> mTransferable;                 // [in]
+
+  // NS_CONTENT_COMMAND_SCROLL
+  // for mScroll.mUnit
+  enum {
+    eCmdScrollUnit_Line,
+    eCmdScrollUnit_Page,
+    eCmdScrollUnit_Whole
+  };
+
+  struct ScrollInfo {
+    ScrollInfo() :
+      mAmount(0), mUnit(eCmdScrollUnit_Line), mIsHorizontal(PR_FALSE)
+    {
+    }
+
+    PRInt32      mAmount;                                  // [in]
+    PRUint8      mUnit;                                    // [in]
+    PRPackedBool mIsHorizontal;                            // [in]
+  } mScroll;
+
   PRPackedBool mOnlyEnabledCheck;                          // [in]
 
   PRPackedBool mSucceeded;                                 // [out]
   PRPackedBool mIsEnabled;                                 // [out]
 };
 
-/**
- * MenuItem event
- * 
- * When this event occurs the widget field in nsGUIEvent holds the "target"
- * for the event
- */
-
-class nsMenuEvent : public nsGUIEvent
+class nsMozTouchEvent : public nsMouseEvent_base
 {
 public:
-  nsMenuEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
-    : nsGUIEvent(isTrusted, msg, w, NS_MENU_EVENT),
-      mMenuItem(nsnull), mCommand(0)
+  nsMozTouchEvent(PRBool isTrusted, PRUint32 msg, nsIWidget* w,
+                  PRUint32 streamIdArg)
+    : nsMouseEvent_base(isTrusted, msg, w, NS_MOZTOUCH_EVENT),
+      streamId(streamIdArg)
   {
   }
 
-  nsIMenuItem * mMenuItem;
-  PRUint32      mCommand;
+  PRUint32 streamId;
 };
 
 /**
@@ -1315,6 +1529,69 @@ public:
   PRFloat64 delta;      // Delta for magnify and rotate events
 };
 
+class nsTransitionEvent : public nsEvent
+{
+public:
+  nsTransitionEvent(PRBool isTrusted, PRUint32 msg,
+                    const nsString &propertyNameArg, float elapsedTimeArg)
+    : nsEvent(isTrusted, msg, NS_TRANSITION_EVENT),
+      propertyName(propertyNameArg), elapsedTime(elapsedTimeArg)
+  {
+  }
+
+  nsString propertyName;
+  float elapsedTime;
+};
+
+#ifdef MOZ_CSS_ANIMATIONS
+class nsAnimationEvent : public nsEvent
+{
+public:
+  nsAnimationEvent(PRBool isTrusted, PRUint32 msg,
+                   const nsString &animationNameArg, float elapsedTimeArg)
+    : nsEvent(isTrusted, msg, NS_ANIMATION_EVENT),
+      animationName(animationNameArg), elapsedTime(elapsedTimeArg)
+  {
+  }
+
+  nsString animationName;
+  float elapsedTime;
+};
+#endif
+
+class nsUIStateChangeEvent : public nsGUIEvent
+{
+public:
+  nsUIStateChangeEvent(PRBool isTrusted, PRUint32 msg, nsIWidget* w)
+    : nsGUIEvent(isTrusted, msg, w, NS_UISTATECHANGE_EVENT),
+      showAccelerators(UIStateChangeType_NoChange),
+      showFocusRings(UIStateChangeType_NoChange)
+  {
+  }
+
+  UIStateChangeType showAccelerators;
+  UIStateChangeType showFocusRings;
+};
+
+/**
+ * Native event pluginEvent for plugins.
+ */
+
+class nsPluginEvent : public nsGUIEvent
+{
+public:
+  nsPluginEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsGUIEvent(isTrusted, msg, w, NS_PLUGIN_EVENT),
+      retargetToFocusedDocument(PR_FALSE)
+  {
+  }
+
+  // If TRUE, this event needs to be retargeted to focused document.
+  // Otherwise, never retargeted.
+  // Defaults to false.
+  PRBool retargetToFocusedDocument;
+};
+
 /**
  * Event status for D&D Event
  */
@@ -1338,8 +1615,13 @@ enum nsDragDropEventStatus {
         ((evnt)->message == NS_MOUSE_ACTIVATE) || \
         ((evnt)->message == NS_MOUSE_ENTER_SYNTH) || \
         ((evnt)->message == NS_MOUSE_EXIT_SYNTH) || \
+        ((evnt)->message == NS_MOUSE_MOZHITTEST) || \
         ((evnt)->message == NS_MOUSE_MOVE))
 
+#define NS_IS_MOUSE_EVENT_STRUCT(evnt) \
+       ((evnt)->eventStructType == NS_MOUSE_EVENT || \
+        (evnt)->eventStructType == NS_DRAG_EVENT)
+
 #define NS_IS_MOUSE_LEFT_CLICK(evnt) \
        ((evnt)->eventStructType == NS_MOUSE_EVENT && \
         (evnt)->message == NS_MOUSE_CLICK && \
@@ -1370,37 +1652,34 @@ enum nsDragDropEventStatus {
 #define NS_IS_IME_EVENT(evnt) \
        (((evnt)->message == NS_TEXT_TEXT) ||  \
         ((evnt)->message == NS_COMPOSITION_START) ||  \
-        ((evnt)->message == NS_COMPOSITION_END) || \
-        ((evnt)->message == NS_COMPOSITION_QUERY))
+        ((evnt)->message == NS_COMPOSITION_END))
 
-#define NS_IS_FOCUS_EVENT(evnt) \
+#define NS_IS_ACTIVATION_EVENT(evnt) \
        (((evnt)->message == NS_ACTIVATE) || \
         ((evnt)->message == NS_DEACTIVATE) || \
-        ((evnt)->message == NS_PLUGIN_ACTIVATE))
+        ((evnt)->message == NS_PLUGIN_ACTIVATE) || \
+        ((evnt)->message == NS_PLUGIN_FOCUS))
 
 #define NS_IS_QUERY_CONTENT_EVENT(evnt) \
-       (((evnt)->message == NS_QUERY_SELECTED_TEXT) || \
-        ((evnt)->message == NS_QUERY_TEXT_CONTENT) || \
-        ((evnt)->message == NS_QUERY_CARET_RECT) || \
-        ((evnt)->message == NS_QUERY_TEXT_RECT) || \
-        ((evnt)->message == NS_QUERY_EDITOR_RECT) || \
-        ((evnt)->message == NS_QUERY_CONTENT_STATE) || \
-        ((evnt)->message == NS_QUERY_SELECTION_AS_TRANSFERABLE) || \
-        ((evnt)->message == NS_QUERY_CHARACTER_AT_POINT))
+       ((evnt)->eventStructType == NS_QUERY_CONTENT_EVENT)
 
 #define NS_IS_SELECTION_EVENT(evnt) \
        (((evnt)->message == NS_SELECTION_SET))
 
 #define NS_IS_CONTENT_COMMAND_EVENT(evnt) \
-       (((evnt)->message == NS_CONTENT_COMMAND_CUT) || \
-        ((evnt)->message == NS_CONTENT_COMMAND_COPY) || \
-        ((evnt)->message == NS_CONTENT_COMMAND_PASTE) || \
-        ((evnt)->message == NS_CONTENT_COMMAND_DELETE) || \
-        ((evnt)->message == NS_CONTENT_COMMAND_UNDO) || \
-        ((evnt)->message == NS_CONTENT_COMMAND_REDO))
+       ((evnt)->eventStructType == NS_CONTENT_COMMAND_EVENT)
 
 #define NS_IS_PLUGIN_EVENT(evnt) \
-       (((evnt)->message == NS_PLUGIN_EVENT))
+       (((evnt)->message == NS_PLUGIN_INPUT_EVENT) || \
+        ((evnt)->message == NS_PLUGIN_FOCUS_EVENT))
+
+#define NS_IS_RETARGETED_PLUGIN_EVENT(evnt) \
+       (NS_IS_PLUGIN_EVENT(evnt) && \
+        (static_cast<nsPluginEvent*>(evnt)->retargetToFocusedDocument))
+
+#define NS_IS_NON_RETARGETED_PLUGIN_EVENT(evnt) \
+       (NS_IS_PLUGIN_EVENT(evnt) && \
+        !(static_cast<nsPluginEvent*>(evnt)->retargetToFocusedDocument))
 
 #define NS_IS_TRUSTED_EVENT(event) \
   (((event)->flags & NS_EVENT_FLAG_TRUSTED) != 0)
@@ -1424,7 +1703,8 @@ enum nsDragDropEventStatus {
 // cases, you should use NS_IS_IME_RELATED_EVENT instead.
 #define NS_IS_IME_RELATED_EVENT(evnt) \
   (NS_IS_IME_EVENT(evnt) || \
-   NS_IS_QUERY_CONTENT_EVENT(evnt) || \
+   (NS_IS_QUERY_CONTENT_EVENT(evnt) && \
+    evnt->message != NS_QUERY_SCROLL_TARGET_INFO) || \
    NS_IS_SELECTION_EVENT(evnt))
 
 /*
@@ -1446,7 +1726,17 @@ enum nsDragDropEventStatus {
 #define NS_VK_ALT            nsIDOMKeyEvent::DOM_VK_ALT
 #define NS_VK_PAUSE          nsIDOMKeyEvent::DOM_VK_PAUSE
 #define NS_VK_CAPS_LOCK      nsIDOMKeyEvent::DOM_VK_CAPS_LOCK
+#define NS_VK_KANA           nsIDOMKeyEvent::DOM_VK_KANA
+#define NS_VK_HANGUL         nsIDOMKeyEvent::DOM_VK_HANGUL
+#define NS_VK_JUNJA          nsIDOMKeyEvent::DOM_VK_JUNJA
+#define NS_VK_FINAL          nsIDOMKeyEvent::DOM_VK_FINAL
+#define NS_VK_HANJA          nsIDOMKeyEvent::DOM_VK_HANJA
+#define NS_VK_KANJI          nsIDOMKeyEvent::DOM_VK_KANJI
 #define NS_VK_ESCAPE         nsIDOMKeyEvent::DOM_VK_ESCAPE
+#define NS_VK_CONVERT        nsIDOMKeyEvent::DOM_VK_CONVERT
+#define NS_VK_NONCONVERT     nsIDOMKeyEvent::DOM_VK_NONCONVERT
+#define NS_VK_ACCEPT         nsIDOMKeyEvent::DOM_VK_ACCEPT
+#define NS_VK_MODECHANGE     nsIDOMKeyEvent::DOM_VK_MODECHANGE
 #define NS_VK_SPACE          nsIDOMKeyEvent::DOM_VK_SPACE
 #define NS_VK_PAGE_UP        nsIDOMKeyEvent::DOM_VK_PAGE_UP
 #define NS_VK_PAGE_DOWN      nsIDOMKeyEvent::DOM_VK_PAGE_DOWN
@@ -1456,6 +1746,9 @@ enum nsDragDropEventStatus {
 #define NS_VK_UP             nsIDOMKeyEvent::DOM_VK_UP
 #define NS_VK_RIGHT          nsIDOMKeyEvent::DOM_VK_RIGHT
 #define NS_VK_DOWN           nsIDOMKeyEvent::DOM_VK_DOWN
+#define NS_VK_SELECT         nsIDOMKeyEvent::DOM_VK_SELECT
+#define NS_VK_PRINT          nsIDOMKeyEvent::DOM_VK_PRINT
+#define NS_VK_EXECUTE        nsIDOMKeyEvent::DOM_VK_EXECUTE
 #define NS_VK_PRINTSCREEN    nsIDOMKeyEvent::DOM_VK_PRINTSCREEN
 #define NS_VK_INSERT         nsIDOMKeyEvent::DOM_VK_INSERT
 #define NS_VK_DELETE         nsIDOMKeyEvent::DOM_VK_DELETE
@@ -1504,6 +1797,7 @@ enum nsDragDropEventStatus {
 #define NS_VK_Z              nsIDOMKeyEvent::DOM_VK_Z
 
 #define NS_VK_CONTEXT_MENU   nsIDOMKeyEvent::DOM_VK_CONTEXT_MENU
+#define NS_VK_SLEEP          nsIDOMKeyEvent::DOM_VK_SLEEP
 
 #define NS_VK_NUMPAD0        nsIDOMKeyEvent::DOM_VK_NUMPAD0
 #define NS_VK_NUMPAD1        nsIDOMKeyEvent::DOM_VK_NUMPAD1
@@ -1567,36 +1861,6 @@ enum nsDragDropEventStatus {
 #define NS_TEXTRANGE_CONVERTEDTEXT         0x04
 #define NS_TEXTRANGE_SELECTEDCONVERTEDTEXT 0x05
 
-inline PRBool NS_TargetUnfocusedEventToLastFocusedContent(nsEvent* aEvent)
-{
-#if defined(MOZ_X11) || defined(XP_MACOSX)
-  // bug 52416 (MOZ_X11)
-  // Lookup region (candidate window) of UNIX IME grabs
-  // input focus from Mozilla but wants to send IME event
-  // to redraw pre-edit (composed) string
-  // If Mozilla does not have input focus and event is IME,
-  // sends IME event to pre-focused element
-
-  // bug 417315 (XP_MACOSX)
-  // The commit event when the window is deactivating is sent after
-  // the next focused widget getting the focus.
-  // We need to send the commit event to last focused content.
-
-  return NS_IS_IME_RELATED_EVENT(aEvent);
-#elif defined(XP_WIN)
-  // bug 292263 (XP_WIN)
-  // If software keyboard has focus, it may send the key messages and
-  // the IME messages to pre-focused window. Therefore, if Mozilla
-  // doesn't have focus and event is key event or IME event, we should
-  // send the events to pre-focused element.
-
-  return NS_IS_KEY_EVENT(aEvent) || NS_IS_IME_RELATED_EVENT(aEvent) ||
-         NS_IS_PLUGIN_EVENT(aEvent) || NS_IS_CONTENT_COMMAND_EVENT(aEvent);
-#else
-  return PR_FALSE;
-#endif
-}
-
 /**
  * Whether the event should be handled by the frame of the mouse cursor
  * position or not.  When it should be handled there (e.g., the mouse events),
@@ -1604,10 +1868,9 @@ inline PRBool NS_TargetUnfocusedEventToLastFocusedContent(nsEvent* aEvent)
  */
 inline PRBool NS_IsEventUsingCoordinates(nsEvent* aEvent)
 {
-  return !NS_IS_KEY_EVENT(aEvent) && !NS_IS_IME_EVENT(aEvent) &&
-         !NS_IS_CONTEXT_MENU_KEY(aEvent) && !NS_IS_FOCUS_EVENT(aEvent) &&
-         !NS_IS_QUERY_CONTENT_EVENT(aEvent) && !NS_IS_PLUGIN_EVENT(aEvent) &&
-         !NS_IS_SELECTION_EVENT(aEvent) &&
+  return !NS_IS_KEY_EVENT(aEvent) && !NS_IS_IME_RELATED_EVENT(aEvent) &&
+         !NS_IS_CONTEXT_MENU_KEY(aEvent) && !NS_IS_ACTIVATION_EVENT(aEvent) &&
+         !NS_IS_PLUGIN_EVENT(aEvent) &&
          !NS_IS_CONTENT_COMMAND_EVENT(aEvent) &&
          aEvent->eventStructType != NS_ACCESSIBLE_EVENT;
 }
@@ -1626,9 +1889,28 @@ inline PRBool NS_IsEventUsingCoordinates(nsEvent* aEvent)
  */
 inline PRBool NS_IsEventTargetedAtFocusedWindow(nsEvent* aEvent)
 {
-  return NS_IS_KEY_EVENT(aEvent) || NS_IS_IME_EVENT(aEvent) ||
-         NS_IS_QUERY_CONTENT_EVENT(aEvent) || NS_IS_SELECTION_EVENT(aEvent) ||
-         NS_IS_CONTEXT_MENU_KEY(aEvent) || NS_IS_CONTENT_COMMAND_EVENT(aEvent);
+  return NS_IS_KEY_EVENT(aEvent) || NS_IS_IME_RELATED_EVENT(aEvent) ||
+         NS_IS_CONTEXT_MENU_KEY(aEvent) ||
+         NS_IS_CONTENT_COMMAND_EVENT(aEvent) ||
+         NS_IS_RETARGETED_PLUGIN_EVENT(aEvent);
+}
+
+/**
+ * Whether the event should be handled by the focused content or not.  E.g.,
+ * key events, IME related events and other input events which are not handled
+ * by the frame of the mouse cursor position.
+ *
+ * NOTE: Even if this returns TRUE, the event isn't going to be handled by the
+ * application level active DOM window which is on another top level window.
+ * So, when the event is fired on a deactive window, the event is going to be
+ * handled by the last focused DOM element of the last focused DOM window in
+ * the last focused window.
+ */
+inline PRBool NS_IsEventTargetedAtFocusedContent(nsEvent* aEvent)
+{
+  return NS_IS_KEY_EVENT(aEvent) || NS_IS_IME_RELATED_EVENT(aEvent) ||
+         NS_IS_CONTEXT_MENU_KEY(aEvent) ||
+         NS_IS_RETARGETED_PLUGIN_EVENT(aEvent);
 }
 
 #endif // nsGUIEvent_h__
