@@ -37,9 +37,11 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsTraceRefcntImpl.h"
+#include "nsXPCOMPrivate.h"
 #include "nscore.h"
 #include "nsISupports.h"
 #include "nsTArray.h"
+#include "prenv.h"
 #include "prprf.h"
 #include "prlog.h"
 #include "plstr.h"
@@ -49,8 +51,23 @@
 #include "nsCRT.h"
 #include <math.h>
 #include "nsStackWalk.h"
+#include "nsString.h"
 
-#ifdef HAVE_LIBDL
+#include "nsXULAppAPI.h"
+#ifdef XP_WIN
+#include <process.h>
+#define getpid _getpid
+#else
+#include <unistd.h>
+#endif
+
+#ifdef NS_TRACE_MALLOC
+#include "nsTraceMalloc.h"
+#endif
+
+#include "mozilla/BlockingResourceBase.h"
+
+#ifdef HAVE_DLOPEN
 #include <dlfcn.h>
 #endif
 
@@ -85,6 +102,8 @@ NS_MeanAndStdDev(double n, double sumOfValues, double sumOfSquaredValues,
 
 #include "prlock.h"
 
+// TraceRefcnt has to use bare PRLock instead of mozilla::Mutex
+// because TraceRefcnt can be used very early in startup.
 static PRLock* gTraceLock;
 
 #define LOCK_TRACELOG()   PR_Lock(gTraceLock)
@@ -138,10 +157,29 @@ struct nsTraceRefcntStats {
 };
 
   // I hope to turn this on for everybody once we hit it a little less.
-#define ASSERT_ACTIVITY_IS_LEGAL                                             \
-  NS_WARN_IF_FALSE(gActivityTLS != BAD_TLS_INDEX &&                          \
-             NS_PTR_TO_INT32(PR_GetThreadPrivate(gActivityTLS)) == 0,        \
-             "XPCOM objects created/destroyed from static ctor/dtor");
+#ifdef DEBUG
+static const char kStaticCtorDtorWarning[] =
+  "XPCOM objects created/destroyed from static ctor/dtor";
+
+static void
+AssertActivityIsLegal()
+{
+  if (gActivityTLS == BAD_TLS_INDEX ||
+      NS_PTR_TO_INT32(PR_GetThreadPrivate(gActivityTLS)) != 0) {
+    if (PR_GetEnv("MOZ_FATAL_STATIC_XPCOM_CTORS_DTORS")) {
+      NS_RUNTIMEABORT(kStaticCtorDtorWarning);
+    } else {
+      NS_WARNING(kStaticCtorDtorWarning);
+    }
+  }
+}
+#  define ASSERT_ACTIVITY_IS_LEGAL              \
+  PR_BEGIN_MACRO                                \
+    AssertActivityIsLegal();                    \
+  PR_END_MACRO
+#else
+#  define ASSERT_ACTIVITY_IS_LEGAL PR_BEGIN_MACRO PR_END_MACRO
+#endif  // DEBUG
 
 // These functions are copied from nsprpub/lib/ds/plhash.c, with changes
 // to the functions not called Default* to free the serialNumberRecord or
@@ -318,8 +356,8 @@ public:
   }
 
   PRBool PrintDumpHeader(FILE* out, const char* msg, nsTraceRefcntImpl::StatisticsType type) {
-    fprintf(out, "\n== BloatView: %s\n", msg);
-
+    fprintf(out, "\n== BloatView: %s, %s process %d\n", msg,
+            XRE_ChildProcessTypeToString(XRE_GetProcessType()), getpid());
     nsTraceRefcntStats& stats =
       (type == nsTraceRefcntImpl::NEW_STATS) ? mNewStats : mAllStats;
     if (gLogLeaksOnly && !HaveLeaks(&stats))
@@ -554,10 +592,6 @@ static PRBool LogThisType(const char* aTypeName)
 
 static PRInt32 GetSerialNumber(void* aPtr, PRBool aCreate)
 {
-#ifdef GC_LEAK_DETECTOR
-  // need to disguise this pointer, so the table won't keep the object alive.
-  aPtr = (void*) ~PLHashNumber(aPtr);
-#endif
   PLHashEntry** hep = PL_HashTableRawLookup(gSerialNumbers, PLHashNumber(NS_PTR_TO_INT32(aPtr)), aPtr);
   if (hep && *hep) {
     return PRInt32((reinterpret_cast<serialNumberRecord*>((*hep)->value))->serialNumber);
@@ -577,10 +611,6 @@ static PRInt32 GetSerialNumber(void* aPtr, PRBool aCreate)
 
 static PRInt32* GetRefCount(void* aPtr)
 {
-#ifdef GC_LEAK_DETECTOR
-  // need to disguise this pointer, so the table won't keep the object alive.
-  aPtr = (void*) ~PLHashNumber(aPtr);
-#endif
   PLHashEntry** hep = PL_HashTableRawLookup(gSerialNumbers, PLHashNumber(NS_PTR_TO_INT32(aPtr)), aPtr);
   if (hep && *hep) {
     return &((reinterpret_cast<serialNumberRecord*>((*hep)->value))->refCount);
@@ -591,10 +621,6 @@ static PRInt32* GetRefCount(void* aPtr)
 
 static PRInt32* GetCOMPtrCount(void* aPtr)
 {
-#ifdef GC_LEAK_DETECTOR
-  // need to disguise this pointer, so the table won't keep the object alive.
-  aPtr = (void*) ~PLHashNumber(aPtr);
-#endif
   PLHashEntry** hep = PL_HashTableRawLookup(gSerialNumbers, PLHashNumber(NS_PTR_TO_INT32(aPtr)), aPtr);
   if (hep && *hep) {
     return &((reinterpret_cast<serialNumberRecord*>((*hep)->value))->COMPtrCount);
@@ -605,10 +631,6 @@ static PRInt32* GetCOMPtrCount(void* aPtr)
 
 static void RecycleSerialNumberPtr(void* aPtr)
 {
-#ifdef GC_LEAK_DETECTOR
-  // need to disguise this pointer, so the table won't keep the object alive.
-  aPtr = (void*) ~PLHashNumber(aPtr);
-#endif
   PL_HashTableRemove(gSerialNumbers, aPtr);
 }
 
@@ -617,6 +639,12 @@ static PRBool LogThisObj(PRInt32 aSerialNumber)
   return nsnull != PL_HashTableLookup(gObjectsToLog, (const void*)(aSerialNumber));
 }
 
+#ifdef XP_WIN
+#define FOPEN_NO_INHERIT "N"
+#else
+#define FOPEN_NO_INHERIT
+#endif
+
 static PRBool InitLog(const char* envVar, const char* msg, FILE* *result)
 {
   const char* value = getenv(envVar);
@@ -634,18 +662,31 @@ static PRBool InitLog(const char* envVar, const char* msg, FILE* *result)
       return PR_TRUE;
     }
     else {
-      FILE *stream = ::fopen(value, "w");
+      FILE *stream;
+      nsCAutoString fname(value);
+      if (XRE_GetProcessType() != GeckoProcessType_Default) {
+        bool hasLogExtension = 
+            fname.RFind(".log", PR_TRUE, -1, 4) == kNotFound ? false : true;
+        if (hasLogExtension)
+          fname.Cut(fname.Length() - 4, 4);
+        fname.AppendLiteral("_");
+        fname.Append((char*)XRE_ChildProcessTypeToString(XRE_GetProcessType()));
+        fname.AppendLiteral("_pid");
+        fname.AppendInt((PRUint32)getpid());
+        if (hasLogExtension)
+          fname.AppendLiteral(".log");
+      }
+      stream = ::fopen(fname.get(), "w" FOPEN_NO_INHERIT);
       if (stream != NULL) {
         *result = stream;
         fprintf(stdout, "### %s defined -- logging %s to %s\n",
-                envVar, msg, value);
-        return PR_TRUE;
+                envVar, msg, fname.get());
       }
       else {
         fprintf(stdout, "### %s defined -- unable to log %s to %s\n",
-                envVar, msg, value);
-        return PR_FALSE;
+                envVar, msg, fname.get());
       }
+      return stream != NULL;
     }
   }
   return PR_FALSE;
@@ -683,7 +724,7 @@ static void InitTraceLog(void)
   if (defined) {
     gLogToLeaky = PR_TRUE;
     PRFuncPtr p = nsnull, q = nsnull;
-#ifdef HAVE_LIBDL
+#ifdef HAVE_DLOPEN
     {
       PRLibrary *lib = nsnull;
       p = PR_FindFunctionSymbolAndLibrary("__log_addref", &lib);
@@ -833,7 +874,7 @@ static void PrintStackFrame(void *aPC, void *aClosure)
 
   NS_DescribeCodeAddress(aPC, &details);
   NS_FormatCodeAddressDetails(aPC, &details, buf, sizeof(buf));
-  fprintf(stream, buf);
+  fputs(buf, stream);
 }
 
 }
@@ -886,15 +927,45 @@ NS_LogInit()
   if (++gInitCount)
     nsTraceRefcntImpl::SetActivityIsLegal(PR_TRUE);
 #endif
+
+#ifdef NS_TRACE_MALLOC
+  // XXX we don't have to worry about shutting down trace-malloc; it
+  // handles this itself, through an atexit() callback.
+  if (!NS_TraceMallocHasStarted())
+    NS_TraceMallocStartup(-1);  // -1 == no logging
+#endif
 }
 
 EXPORT_XPCOM_API(void)
 NS_LogTerm()
+{
+  mozilla::LogTerm();
+}
+
+namespace mozilla {
+void
+LogTerm()
 {
   NS_ASSERTION(gInitCount > 0,
                "NS_LogTerm without matching NS_LogInit");
 
   if (--gInitCount == 0) {
+#ifdef DEBUG
+    /* FIXME bug 491977: This is only going to operate on the
+     * BlockingResourceBase which is compiled into
+     * libxul/libxpcom_core.so. Anyone using external linkage will
+     * have their own copy of BlockingResourceBase statics which will
+     * not be freed by this method.
+     *
+     * It sounds like what we really want is to be able to register a
+     * callback function to call at XPCOM shutdown.  Note that with
+     * this solution, however, we need to guarantee that
+     * BlockingResourceBase::Shutdown() runs after all other shutdown
+     * functions.
+     */
+    BlockingResourceBase::Shutdown();
+#endif
+    
     if (gInitialized) {
       nsTraceRefcntImpl::DumpStatistics();
       nsTraceRefcntImpl::ResetStatistics();
@@ -907,6 +978,8 @@ NS_LogTerm()
   }
 }
 
+} // namespace mozilla
+
 EXPORT_XPCOM_API(void)
 NS_LogAddRef(void* aPtr, nsrefcnt aRefcnt,
              const char* aClazz, PRUint32 classSize)
@@ -925,7 +998,7 @@ NS_LogAddRef(void* aPtr, nsrefcnt aRefcnt,
       }
     }
 
-    // Here's the case where neither NS_NEWXPCOM nor MOZ_COUNT_CTOR were used,
+    // Here's the case where MOZ_COUNT_CTOR was not used,
     // yet we still want to see creation information:
 
     PRBool loggingThisType = (!gTypesToLog || LogThisType(aClazz));
@@ -1009,7 +1082,7 @@ NS_LogRelease(void* aPtr, nsrefcnt aRefcnt, const char* aClazz)
       }
     }
 
-    // Here's the case where neither NS_DELETEXPCOM nor MOZ_COUNT_DTOR were used,
+    // Here's the case where MOZ_COUNT_DTOR was not used,
     // yet we still want to see deletion information:
 
     if (aRefcnt == 0 && gAllocLog && loggingThisType && loggingThisObject) {
